from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks

_CITATION = """\
"""

_LANGUAGES = ["ind"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_LOCAL = False

_DATASETNAME = "netifier"

_DESCRIPTION = """\
Netifier dataset is a collection of scraped posts on famous social media sites in Indonesia,
such as Instagram, Twitter, and Kaskus aimed to do multi-label toxicity classification.
The dataset consists of 7,773 texts. The author manually labelled ~7k samples into 4 categories:
pornography, hate speech, racism, and radicalism.
"""

_HOMEPAGE = "https://github.com/ahmadizzan/netifier"
_LICENSE = "Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International"
_URLS = {_DATASETNAME: {"train": "https://raw.githubusercontent.com/ahmadizzan/netifier/master/data/processed/train.csv", "test": "https://raw.githubusercontent.com/ahmadizzan/netifier/master/data/processed/test.csv"}}
_SUPPORTED_TASKS = [Tasks.ASPECT_BASED_SENTIMENT_ANALYSIS]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class Netifier(datasets.GeneratorBasedBuilder):
    """Netifier dataset is a collection of scraped posts on famous social media sites in Indonesia,
    such as Instagram, Twitter, and Kaskus aimed to do multi-label toxicity classification."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="netifier_source",
            version=SOURCE_VERSION,
            description="Netifier source schema",
            schema="source",
            subset_id="netifier",
        ),
        SEACrowdConfig(
            name="netifier_seacrowd_text_multi",
            version=SEACROWD_VERSION,
            description="Netifier Nusantara schema",
            schema="seacrowd_text_multi",
            subset_id="netifier",
        ),
    ]

    DEFAULT_CONFIG_NAME = "netifier_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "text": datasets.Value("string"),
                    "pornography": datasets.Value("bool"),
                    "blasphemy_racism_discrimination": datasets.Value("bool"),
                    "radicalism": datasets.Value("bool"),
                    "defamation": datasets.Value("bool"),
                }
            )
        elif self.config.schema == "seacrowd_text_multi":
            features = schemas.text_multi_features([0, 1])

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        urls = _URLS[_DATASETNAME]
        train_data = Path(dl_manager.download(urls["train"]))
        test_data = Path(dl_manager.download(urls["test"]))

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": train_data,
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": test_data,
                    "split": "test",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        # Dataset does not have id, using row index as id
        label_cols = ["pornography", "blasphemy_racism_discrimination", "radicalism", "defamation"]
        df = pd.read_csv(filepath, encoding="ISO-8859-1").reset_index()
        df.columns = ["id", "original_text", "text"] + label_cols

        if self.config.schema == "source":
            for row in df.itertuples():
                ex = {
                    "text": row.text,
                }
                for label in label_cols:
                    ex[label] = getattr(row, label)
                yield row.id, ex

        elif self.config.schema == "seacrowd_text_multi":
            for row in df.itertuples():
                ex = {
                    "id": str(row.id),
                    "text": row.text,
                    "labels": [label for label in row[4:]],
                }
                yield row.id, ex
        else:
            raise ValueError(f"Invalid config: {self.config.name}")



"""
Self-supervised Pretraining (SSP) Classification Schema
"""
import datasets

features = datasets.Features(
    {
        "id": datasets.Value("string"),
        "text": datasets.Value("string"),
    }
)


import os
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@Article{app12084083,
AUTHOR = {Onuean, Athita and Buatoom, Uraiwan and Charoenporn, Thatsanee and Kim, Taehong and Jung, Hanmin},
TITLE = {Burapha-TH: A Multi-Purpose Character, Digit, and Syllable Handwriting Dataset},
JOURNAL = {Applied Sciences},
VOLUME = {12},
YEAR = {2022},
NUMBER = {8},
ARTICLE-NUMBER = {4083},
URL = {https://www.mdpi.com/2076-3417/12/8/4083},
ISSN = {2076-3417},
DOI = {10.3390/app12084083}
}
"""
_DATASETNAME = "burapha_th"

_DESCRIPTION = """\
The dataset has 68 character classes, 10 digit classes, and 320 syllable classes.
For constructing the dataset, 1072 Thai native speakers wrote on collection datasheets
that were then digitized using a 300 dpi scanner.
De-skewing, detection box and segmentation algorithms were applied to the raw scans
for image extraction. The dataset, unlike all other known Thai handwriting datasets, retains
existing noise, the white background, and all artifacts generated by scanning.
"""

_HOMEPAGE = "https://services.informatics.buu.ac.th/datasets/Burapha-TH/"

_LICENSE = Licenses.UNKNOWN.value

_LOCAL = False
_LANGUAGES = ["tha"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)

_URLS = {
    "character": {"test": "https://services.informatics.buu.ac.th/datasets/Burapha-TH/character/20210306-test.zip", "train": "https://services.informatics.buu.ac.th/datasets/Burapha-TH/character/20210306-train.zip"},
    "digit": {"test": "https://services.informatics.buu.ac.th/datasets/Burapha-TH/digit/20210307-test.zip", "train": "https://services.informatics.buu.ac.th/datasets/Burapha-TH/digit/20210307-train.zip"},
    "syllable": {"test": "https://services.informatics.buu.ac.th/datasets/Burapha-TH/syllable/20210309-test-ori.zip", "train": "https://services.informatics.buu.ac.th/datasets/Burapha-TH/syllable/20210309-train-ori.zip"},
}

_SUPPORTED_TASKS = [Tasks.IMAGE_CAPTIONING]
_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"

_SUBSETS = ["character", "digit", "syllable"]


def config_constructor(subset: str, schema: str, version: str) -> SEACrowdConfig:
    return SEACrowdConfig(
        name=f"{_DATASETNAME}_{subset}_{schema}",
        version=version,
        description=f"{_DATASETNAME} {subset} {schema} schema",
        schema=f"{schema}",
        subset_id=f"{_DATASETNAME}_{subset}",
    )


class BuraphaThDataset(datasets.GeneratorBasedBuilder):
    """
    The dataset has 68 character classes, 10 digit classes, and 320 syllable classes.
    For constructing the dataset, 1072 Thai native speakers wrote on collection datasheets
    that were then digitized using a 300 dpi scanner.
    De-skewing, detection box and segmentation algorithms were applied to the raw scans for
    image extraction. The dataset, unlike all other known Thai handwriting datasets, retains
    existing noise, the white background, and all artifacts generated by scanning.
    """

    BUILDER_CONFIGS = [config_constructor(subset, "source", _SOURCE_VERSION) for subset in _SUBSETS]
    BUILDER_CONFIGS.extend([config_constructor(subset, "seacrowd_imtext", _SEACROWD_VERSION) for subset in _SUBSETS])

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_digit_source"

    label_chr_dig = [str(i).zfill(2) for i in range(78)]
    label_syl = [str(i).zfill(3) for i in range(320)]

    def _info(self) -> datasets.DatasetInfo:
        task = self.config.subset_id.split("_")[2]
        if self.config.schema == "source":
            features = datasets.Features(
                {"id": datasets.Value("string"), "image_paths": datasets.Value("string"), "label": datasets.Sequence(datasets.ClassLabel(names=self.label_chr_dig if task == "character" or task == "digit" else self.label_syl))}
            )
        elif self.config.schema == "seacrowd_imtext":
            features = schemas.image_text_features(label_names=self.label_chr_dig if task == "character" or task == "digit" else self.label_syl)
        else:
            raise NotImplementedError()

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""

        task = self.config.subset_id.split("_")[2]

        _local_path = dl_manager.download_and_extract(_URLS[task])
        train_path, test_path = _local_path["train"], _local_path["test"]
        if task in ["character", "digit"]:
            train_path = os.path.join(train_path, "train")
            test_path = os.path.join(test_path, "test")
        # for "syllable" type task
        else:
            train_path = os.path.join(train_path, "train-ori")
            test_path = os.path.join(test_path, "test-ori")

        data_pair = {}

        for dir_name in os.listdir(train_path):
            dir_name_split = dir_name.split("-")
            file_names = []

            for file_name in os.listdir(os.path.join(train_path, dir_name)):
                file_names.append(os.path.join(train_path, dir_name, file_name))

            label = dir_name_split[0]
            data_pair[label] = file_names

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_pair,
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": data_pair,
                    "split": "test",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        task = self.config.subset_id.split("_")[2]
        counter = 0

        for key, imgs in filepath.items():
            for img in imgs:
                if self.config.schema == "source":
                    yield counter, {"id": str(counter), "image_paths": img, "label": [self.label_chr_dig.index(key) if task == "character" or task == "digit" else self.label_syl.index(key)]}
                elif self.config.schema == "seacrowd_imtext":
                    yield counter, {
                        "id": str(counter),
                        "image_paths": [img],
                        "texts": None,
                        "metadata": {
                            "context": None,
                            "labels": [self.label_chr_dig.index(key) if task in ["character", "digit"] else self.label_syl.index(key)],
                        },
                    }
                counter += 1



"""
Question Answering Schema
"""
import datasets

features = datasets.Features(
    {
        "id": datasets.Value("string"),
        "question_id": datasets.Value("string"),
        "document_id": datasets.Value("string"),
        "question": datasets.Value("string"),
        "type": datasets.Value("string"),
        "choices": datasets.Sequence(datasets.Value("string")),
        "context": datasets.Value("string"),
        "answer": datasets.Sequence(datasets.Value("string")),

        # the schema of 'meta' aren't specified either to allow some flexibility
        "meta": {}

        # notes on how to use this field of 'meta'
        # you can choose two of options:
        # 1. defining as empty dict if you don't think it's usable in `_generate_examples`, or
        # 2. defining meta as dict of key with intended colname meta and its val with dataset.Features class
        #    in `_info` Dataloader method then populate it with the values in `_general_examples` Dataloader method
    }
)



import re

def listToString(list_of_string) -> str:
    one_string = " ".join(list_of_string)
    one_string = re.sub(r" (-\/) ", r"\1", one_string)
    one_string = re.sub(r" ([:;?!%.,])", r"\1", one_string)
    one_string = re.sub(r"\( ([^)]+) \)", r"(\1)", one_string)
    one_string = re.sub(r"\' ([^\']+) \'", r"'\1'", one_string)
    one_string = re.sub(r"\" ([^\"]+) \"", r'"\1"', one_string)
    return one_string

def getAnswerString(list_of_string, answer_mask) -> str:
    answer_string = ""
    for i in range(len(answer_mask)):
        if (answer_mask[i] == 'B'):
            if(answer_string != ""):
                answer_string += ", "
            answer_string += list_of_string[i]
        elif (answer_mask[i] == 'I'):
            answer_string += " " + list_of_string[i]

    return answer_string


from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks

_CITATION = """\
@inproceedings{inproceedings,
author = {Alfina, Ika and Mulia, Rio and Fanany, Mohamad Ivan and Ekanata, Yudo},
year = {2017},
month = {10},
pages = {},
title = {Hate Speech Detection in the Indonesian Language: A Dataset and Preliminary Study},
doi = {10.1109/ICACSIS.2017.8355039}
}
"""

_LOCAL = False
_LANGUAGES = ["ind"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_DATASETNAME = "id_hatespeech"

_DESCRIPTION = """\
The ID Hatespeech dataset is collection of 713 tweets related to a political event, the Jakarta Governor Election 2017
designed for hate speech detection NLP task. This dataset is crawled from Twitter, and then filtered
and annotated manually. The dataset labelled into two; HS if the tweet contains hate speech and Non_HS if otherwise
"""

_HOMEPAGE = "https://www.researchgate.net/publication/320131169_Hate_Speech_Detection_in_the_Indonesian_Language_A_Dataset_and_Preliminary_Study"
_LICENSE = "Unknown"
_URLS = {
    _DATASETNAME: "https://raw.githubusercontent.com/ialfina/id-hatespeech-detection/master/IDHSD_RIO_unbalanced_713_2017.txt",
}
_SUPPORTED_TASKS = [Tasks.SENTIMENT_ANALYSIS]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class IdHatespeech(datasets.GeneratorBasedBuilder):
    """The ID Hatespeech dataset is collection of tweets related to a political event, the Jakarta Governor Election 2017
    designed for hate speech detection NLP task."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="id_hatespeech_source",
            version=SOURCE_VERSION,
            description="ID Hatespeech source schema",
            schema="source",
            subset_id="id_hatespeech",
        ),
        SEACrowdConfig(
            name="id_hatespeech_seacrowd_text",
            version=SEACROWD_VERSION,
            description="ID Hatespeech Nusantara schema",
            schema="seacrowd_text",
            subset_id="id_hatespeech",
        ),
    ]

    DEFAULT_CONFIG_NAME = "id_hatespeech_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features({"tweet": datasets.Value("string"), "label": datasets.Value("string")})
        elif self.config.schema == "seacrowd_text":
            features = schemas.text_features(["Non_HS", "HS"])
   

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        # Dataset does not have predetermined split, putting all as TRAIN
        urls = _URLS[_DATASETNAME]
        base_dir = Path(dl_manager.download_and_extract(urls))
        data_files = {"train": base_dir}

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_files["train"],
                    "split": "train",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        # Dataset does not have id, using row index as id
        df = pd.read_csv(filepath, sep="\t", encoding="ISO-8859-1").reset_index()
        df.columns = ["id", "label", "tweet"]

        if self.config.schema == "source":
            for row in df.itertuples():
                ex = {
                    "tweet": row.tweet,
                    "label": row.label,
                }
                yield row.id, ex

        elif self.config.schema == "seacrowd_text":
            for row in df.itertuples():
                ex = {
                    "id": str(row.id),
                    "text": row.tweet,
                    "label": row.label,
                }
                yield row.id, ex
        else:
            raise ValueError(f"Invalid config: {self.config.name}")



"""
SEA Crowd Data Loader for Bloom Captioning.
"""
from typing import Dict, List, Tuple

import datasets
from datasets.download.download_manager import DownloadManager

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import TASK_TO_SCHEMA, Licenses, Tasks

_CITATION = r"""
@inproceedings{leong-etal-2022-bloom,
    title = "Bloom Library: Multimodal Datasets in 300+ Languages for a Variety of Downstream Tasks",
    author = "Leong, Colin  and
      Nemecek, Joshua  and
      Mansdorfer, Jacob  and
      Filighera, Anna  and
      Owodunni, Abraham  and
      Whitenack, Daniel",
    editor = "Goldberg, Yoav  and
      Kozareva, Zornitsa  and
      Zhang, Yue",
    booktitle = "Proceedings of the 2022 Conference on Empirical Methods in Natural Language Processing",
    month = dec,
    year = "2022",
    address = "Abu Dhabi, United Arab Emirates",
    publisher = "Association for Computational Linguistics",
    url = "https://aclanthology.org/2022.emnlp-main.590",
    doi = "10.18653/v1/2022.emnlp-main.590",
    pages = "8608--8621",
}
"""

logger = datasets.logging.get_logger(__name__)

# this config is created for SEACrowd Dataloader
_LANG_CONFIG = {
    "abc": "Ambala Ayta",
    "ahk": "Akha",
    "bfn": "Bunak",
    "bjn": "Banjar",
    "bkx": "Baikeno",
    "brb": "Brao",
    "brv": "Western Bru",
    "bya": "Batak",
    "bzi": "Bisu",
    "ceb": "Cebuano",
    "cgc": "Kagayanen",
    "cmo": "Central Mnong",
    "ddg": "Fataluku",
    "dmg": "Upper Kinabatangan",
    "dnw": "Western Dani",
    "dtp": "Kadazan Dusun",
    "dtr": "Lotud",
    "enc": "En",
    "fil": "Filipino",
    "gal": "Galolen",
    "hil": "Hiligaynon",
    "hre": "Hre",
    "hro": "Haroi",
    "idt": "Idaté",
    "ilo": "Ilocano",
    "ind": "Indonesian",
    "jra": "Jarai",
    "kak": "Kalanguya",
    "khb": "Lü",
    "khm": "Khmer",
    "kqr": "Kimaragang",
    "krr": "Krung",
    "ksw": "S’gaw Karen",
    "lhu": "Lahu",
    "llg": "Lole",
    "lsi": "Lacid",
    "lwl": "Eastern Lawa",
    "mdr": "Mandar",
    "mgm": "Mambae",
    "mhx": "Lhao Vo",
    "mkz": "Makasae",
    "mnw": "Mon",
    "mqj": "Mamasa",
    "mry": "Mandaya",
    "msb": "Masbatenyo",
    "mya": "Burmese",
    "nod": "Northern Thai",
    "nst": "Tangshang Naga",
    "nxa": "Nauete",
    "nxl": "South Nuaulu",
    "pag": "Pangasinan",
    "pce": "Ruching Palaung",
    "pdu": "Kayan",
    "pea": "Peranakan Indonesian",
    "pmf": "Pamona",
    "sea": "Semai",
    "sgd": "Surigaonon",
    "shn": "Shan",
    "sml": "Central Sama",
    "snl": "Sangil",
    "tdt": "Tetun Dili",
    "tet": "Tetun",
    "tha": "Thai",
    "tkd": "Tukudede",
    "tnt": "Tontemboan",
    "tom": "Tombulu",
    "tpu": "Tampuan",
    "vie": "Vietnamese",
    "war": "Waray-Waray",
    "wms": "Wambon",
    "wnk": "Wanukaka",
    "xmm": "Manado Malay",
    "yet": "Yetfa",
    "zlm": "Malay",
}

_LOCAL = False
_LANGUAGES = list(_LANG_CONFIG.keys())


_DATASETNAME = "bloom_captioning"
_DESCRIPTION = r"""
This is a Bloom Library dataset developed for the image captioning task.
It covers 74 languages indigenous to SEA overall, amounting to total data of 21K.
This dataset belongs to a CC license, where its datapoints has specific license attached to it.
Before using this dataloader, please accept the acknowledgement at https://huggingface.co/datasets/sil-ai/bloom-captioning and use huggingface-cli login for authentication.
"""

_HOMEPAGE = "https://huggingface.co/datasets/sil-ai/bloom-captioning"
_LICENSE = Licenses.CC.value

_URL = "https://huggingface.co/datasets/sil-ai/bloom-captioning"
_HF_REMOTE_REF = "/".join(_URL.split("/")[-2:])

_SUPPORTED_TASKS = [Tasks.IMAGE_CAPTIONING]
_SOURCE_VERSION = "0.1.0"
_SEACROWD_VERSION = "1.0.0"

CONFIG_SUFFIXES_FOR_TASK = [TASK_TO_SCHEMA.get(task).lower() for task in _SUPPORTED_TASKS]


def construct_configs_on_langs(languages: list = None) -> List[SEACrowdConfig]:
    """
    The function `construct_configs` constructs a list of SEACrowdConfig objects based on the provided
    languages or a default language, and returns the list.

    input:
        languages (list, default None): The `languages` parameter is a list that specifies the languages for which the
        configurations need to be constructed. If no languages are provided (value=None), the first value in language config
        will be used.
    output:
        a list of `SEACrowdConfig` objects based on instantiated init variables
    """

    # set output var
    config_list = []

    # construct zipped arg for config instantiation
    TASKS_AND_CONFIG_SUFFIX_PAIRS = list(zip(_SUPPORTED_TASKS, CONFIG_SUFFIXES_FOR_TASK))

    # implement source schema
    version, config_name_prefix = _SOURCE_VERSION, "source"
    config_list += [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_{_LANG}_{config_name_prefix}",
            version=datasets.Version(version),
            description=f"{_DATASETNAME} {config_name_prefix} schema for language code {_LANG}",
            schema=f"{config_name_prefix}",
            subset_id=_LANG,
        )
        for _LANG in languages
    ]

    # implement SEACrowd schema
    version, config_name_prefix = _SEACROWD_VERSION, "seacrowd"
    for task_obj, config_name_suffix in TASKS_AND_CONFIG_SUFFIX_PAIRS:
        config_list += [
            SEACrowdConfig(
                name=f"{_DATASETNAME}_{_LANG}_{config_name_prefix}_{config_name_suffix}",
                version=datasets.Version(version),
                description=f"{_DATASETNAME} {config_name_prefix} schema for {task_obj.name} and language code {_LANG}",
                schema=f"{config_name_prefix}_{config_name_suffix}",
                subset_id=_LANG,
            )
            for _LANG in languages
        ]
    return config_list


class BloomCaptioningDataset(datasets.GeneratorBasedBuilder):
    """Bloom Captioning dataset, subsetted from https://huggingface.co/datasets/sil-ai/bloom-captioning"""

    # get all schema w/o lang arg + get all schema w/ lang arg
    BUILDER_CONFIGS = construct_configs_on_langs(_LANGUAGES)

    def _info(self) -> datasets.DatasetInfo:
        _config_schema_name = self.config.schema
        logger.info(f"Received schema name: {self.config.schema}")
        # source schema
        if _config_schema_name == "source":
            features = datasets.Features(
                {
                    "image_id": datasets.Value("string"),
                    "image_url": datasets.Value("string"),
                    "caption": datasets.Value("string"),
                    "story_id": datasets.Value("string"),
                    "album_id": datasets.Value("string"),
                    "license": datasets.Value("string"),
                    "original_bloom_language_tag": datasets.Value("string"),
                    "index_in_story": datasets.Value("uint16"),
                }
            )

        # image-text schema
        elif _config_schema_name == "seacrowd_imtext":
            features = schemas.image_text_features()

        else:
            raise ValueError(f"Received unexpected config schema of {_config_schema_name}!")

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: DownloadManager) -> List[datasets.SplitGenerator]:
        hf_dset_dict = datasets.load_dataset(_HF_REMOTE_REF, self.config.subset_id)

        return [datasets.SplitGenerator(name=datasets.Split(dset_key), gen_kwargs={"hf_dset": dset}) for dset_key, dset in hf_dset_dict.items() if dset.num_rows > 0]

    def _generate_examples(self, hf_dset) -> Tuple[int, Dict]:
        _config_schema_name = self.config.schema

        _idx = 0
        for datapoints in hf_dset:
            # the `_idx` will be generated manually since no `id` present in the dataset fulfill the purpose as primary key
            if _config_schema_name == "source":
                yield _idx, {colname: datapoints[colname] for colname in self.info.features}

            elif _config_schema_name == "seacrowd_imtext":
                yield _idx, {"id": _idx, "image_paths": [datapoints["image_url"]], "texts": datapoints["caption"], "metadata": {"context": "", "labels": []}}

            else:
                raise ValueError(f"Received unexpected config schema of {_config_schema_name}!")

            _idx += 1



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@inproceedings{nguyen-etal-2023-visobert,
    title = "{V}i{S}o{BERT}: A Pre-Trained Language Model for {V}ietnamese Social Media Text Processing",
    author = "Nguyen, Nam  and
      Phan, Thang  and
      Nguyen, Duc-Vu  and
      Nguyen, Kiet",
    editor = "Bouamor, Houda  and
      Pino, Juan  and
      Bali, Kalika",
    booktitle = "Proceedings of the 2023 Conference on Empirical Methods in Natural Language Processing",
    month = dec,
    year = "2023",
    address = "Singapore",
    publisher = "Association for Computational Linguistics",
    url = "https://aclanthology.org/2023.emnlp-main.315",
    pages = "5191--5207",
    abstract = "English and Chinese, known as resource-rich languages, have witnessed the strong
    development of transformer-based language models for natural language processing tasks. Although
    Vietnam has approximately 100M people speaking Vietnamese, several pre-trained models, e.g., PhoBERT,
    ViBERT, and vELECTRA, performed well on general Vietnamese NLP tasks, including POS tagging and
    named entity recognition. These pre-trained language models are still limited to Vietnamese social
    media tasks. In this paper, we present the first monolingual pre-trained language model for
    Vietnamese social media texts, ViSoBERT, which is pre-trained on a large-scale corpus of high-quality
    and diverse Vietnamese social media texts using XLM-R architecture. Moreover, we explored our
    pre-trained model on five important natural language downstream tasks on Vietnamese social media
    texts: emotion recognition, hate speech detection, sentiment analysis, spam reviews detection, and
    hate speech spans detection. Our experiments demonstrate that ViSoBERT, with far fewer parameters,
    surpasses the previous state-of-the-art models on multiple Vietnamese social media tasks. Our
    ViSoBERT model is available only for research purposes. Disclaimer: This paper contains actual
    comments on social networks that might be construed as abusive, offensive, or obscene.",
}
"""

_DATASETNAME = "visobert"

_DESCRIPTION = """\
The ViSoBERT corpus is composed of Vietnamese textual data crawled from Facebook, TikTok, and YouTube. The
dataset contains Facebook posts, TikTok comments, and Youtube comments of Vietnamese-verified users, from
Jan 2016 (Jan 2020 for TikTok) to Dec 2022. A post-processing mechanism is applied to handles hashtags,
emojis, misspellings, hyperlinks, and other noncanonical texts.
"""

_HOMEPAGE = "https://huggingface.co/uitnlp/visobert"

_LANGUAGES = ["vie"]

_LICENSE = Licenses.CC_BY_NC_4_0.value

_LOCAL = False

_URLS = "https://drive.usercontent.google.com/download?id=1BoiR9k2DrjBcd2aHy5BOq4haEp5V2_ug&confirm=xxx"

_SUPPORTED_TASKS = [Tasks.SELF_SUPERVISED_PRETRAINING]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class ViSoBERTDataset(datasets.GeneratorBasedBuilder):
    """
    The ViSoBERT corpus is a Vietnamese pretraining dataset from https://huggingface.co/uitnlp/visobert.
    """

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=datasets.Version(_SOURCE_VERSION),
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_ssp",
            version=datasets.Version(_SEACROWD_VERSION),
            description=f"{_DATASETNAME} SEACrowd schema",
            schema="seacrowd_ssp",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source" or self.config.schema == "seacrowd_ssp":
            features = schemas.self_supervised_pretraining.features
        else:
            raise ValueError(f"Invalid schema: '{self.config.schema}'")

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """
        Returns SplitGenerators.
        """

        path = dl_manager.download(_URLS)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": path,
                    "split": "train",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """
        Yields examples as (key, example) tuples.
        """

        with open(filepath, "r", encoding="utf-8") as f:
            if self.config.schema == "source" or self.config.schema == "seacrowd_ssp":
                for idx, row in enumerate(f):
                    if row.strip() != "":
                        yield (
                            idx,
                            {
                                "id": str(idx),
                                "text": row.strip(),
                            },
                        )
            else:
                raise ValueError(f"Invalid config: '{self.config.name}'")


# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
The Madurese Parallel Corpus Dataset is created by scraping content from the online Bible, resulting in 30,013 Indonesian-Madurese sentences.
This corpus is distinct from a previous Madurese dataset, which was gathered from physical documents such as the Kamus Lengkap Bahasa Madura-Indonesia.
The proposed dataset provides bilingual sentences, allowing for comparisons between Indonesian and Madurese. It aims to supplement existing Madurese
corpora, enabling enhanced research and development focused on regional languages in Indonesia. Unlike the prior dataset that included information
like lemmas, pronunciation, linguistic descriptions, part of speech, loanwords, dialects, and various structures, this new corpus primarily focuses
on bilingual sentence pairs, potentially broadening the scope for linguistic studies and language technology advancements in the Madurese language.
"""
import os
from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import jsonlines

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@article{,
  author    = {Sulistyo, Danang Arbian and Wibawa, Aji Prasetya and Prasetya, Didik Dwi and Nafalski, Andrew},
  title     = {Autogenerated Indonesian-Madurese Parallel Corpus Dataset Using Neural Machine Translation},
  journal   = {Available at SSRN 4644430},
  volume    = {},
  year      = {2023},
  url       = {https://papers.ssrn.com/sol3/papers.cfm?abstract_id=4644430},
  doi       = {},
  biburl    = {},
  bibsource = {}
}
"""

_DATASETNAME = "indonesian_madurese_bible_translation"

_DESCRIPTION = """\
The Madurese Parallel Corpus Dataset is created by scraping content from the online Bible, resulting in 30,013 Indonesian-Madurese sentences.
This corpus is distinct from a previous Madurese dataset, which was gathered from physical documents such as the Kamus Lengkap Bahasa Madura-Indonesia.
The proposed dataset provides bilingual sentences, allowing for comparisons between Indonesian and Madurese. It aims to supplement existing Madurese
corpora, enabling enhanced research and development focused on regional languages in Indonesia. Unlike the prior dataset that included information
like lemmas, pronunciation, linguistic descriptions, part of speech, loanwords, dialects, and various structures, this new corpus primarily focuses
on bilingual sentence pairs, potentially broadening the scope for linguistic studies and language technology advancements in the Madurese language.
"""

_HOMEPAGE = "https://data.mendeley.com/datasets/cgtg4bhrtf/3"
_LANGUAGES = ["ind", "mad"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_LICENSE = Licenses.CC_BY_4_0.value  # example: Licenses.MIT.value, Licenses.CC_BY_NC_SA_4_0.value, Licenses.UNLICENSE.value, Licenses.UNKNOWN.value
_LOCAL = False
_URLS = {
    _DATASETNAME: "https://prod-dcd-datasets-cache-zipfiles.s3.eu-west-1.amazonaws.com/cgtg4bhrtf-3.zip",
}
_SUPPORTED_TASKS = [Tasks.MACHINE_TRANSLATION]  # example: [Tasks.TRANSLITERATION, Tasks.NAMED_ENTITY_RECOGNITION, Tasks.RELATION_EXTRACTION]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class IndonesianMadureseBibleTranslationDataset(datasets.GeneratorBasedBuilder):
    """TODO: This corpus consists of more than 20,000 Indonesian - Madurese sentences."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_t2t",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema="seacrowd_t2t",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    DEFAULT_CONFIG_NAME = "indonesian_madurese_bible_translation_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "src": datasets.Value("string"),
                    "tgt": datasets.Value("string"),
                }
            )

        elif self.config.schema == "seacrowd_t2t":
            features = schemas.text2text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""

        urls = _URLS[_DATASETNAME]
        data_dir = dl_manager.download_and_extract(urls)
        data_dir = os.path.join(data_dir, "Bahasa Madura Corpus Dataset/Indonesian-Madurese Corpus")
        all_raw_path = [data_dir + "/" + item for item in os.listdir(data_dir)]
        all_path = []
        id = 0
        for raw_path in all_raw_path:
            if "txt" in raw_path:
                all_path.append(raw_path)
        all_data = []
        for path in all_path:
            data = self._read_txt(path)
            for line in data:
                if line != "\n":
                    all_data.append({"src": line.split("\t")[0], "tgt": line.split("\t")[1], "id": id})
                id += 1
        self._write_jsonl(data_dir + "/train.jsonl", all_data)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # Whatever you put in gen_kwargs will be passed to _generate_examples
                gen_kwargs={
                    "filepath": os.path.join(data_dir, "train.jsonl"),
                    "split": "train",
                },
            )
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        if self.config.schema == "source":
            i = 0
            with jsonlines.open(filepath) as f:
                for each_data in f.iter():
                    ex = {
                        "id": each_data["id"],
                        "src": each_data["src"],
                        "tgt": each_data["tgt"],
                    }
                    yield i, ex
                    i += 1

        elif self.config.schema == "seacrowd_t2t":
            i = 0
            with jsonlines.open(filepath) as f:
                for each_data in f.iter():
                    ex = {"id": each_data["id"], "text_1": each_data["src"].strip(), "text_2": each_data["tgt"].strip(), "text_1_name": "ind", "text_2_name": "mad"}
                    yield i, ex
                    i += 1

    def _write_jsonl(self, filepath, values):
        with jsonlines.open(filepath, "w") as writer:
            for line in values:
                writer.write(line)

    def _read_txt(self, filepath):
        with open(filepath, "r") as f:
            lines = f.readlines()
        return lines



from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses

_CITATION = """\
@InProceedings{NOMOTO18.8,
  author = {Hiroki Nomoto ,Hannah Choi ,David Moeljadi and Francis Bond},
  title = {MALINDO Morph: Morphological dictionary and analyser for Malay/Indonesian},
  booktitle = {Proceedings of the Eleventh International Conference on Language Resources and Evaluation (LREC 2018)},
  year = {2018},
  month = {may},
  date = {7-12},
  location = {Miyazaki, Japan},
  editor = {Kiyoaki Shirai},
  publisher = {European Language Resources Association (ELRA)},
  address = {Paris, France},
  isbn = {979-10-95546-24-5},
  language = {english}
  }
"""


_DATASETNAME = "malindo_morph"

_DESCRIPTION = """\
MALINDO Morph is a morphological dictionary for Malay (bahasa Melayu) and Indonesian (bahasa Indonesia) language.
It contains over 200,000 lines, with each containing an analysis for one (case-sensitive) token.
Each line is made up of the following six items, separated by tabs: root, surface form, prefix, suffix, circumfix, reduplication.
"""

_HOMEPAGE = "https://github.com/matbahasa/MALINDO_Morph"

_LANGUAGES = ["zlm", "ind"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)

_LICENSE = Licenses.CC_BY_4_0.value  # example: Licenses.MIT.value, Licenses.CC_BY_NC_SA_4_0.value, Licenses.UNLICENSE.value, Licenses.UNKNOWN.value

_LOCAL = False

_URLS = {
    _DATASETNAME: "https://raw.githubusercontent.com/matbahasa/MALINDO_Morph/master/malindo_dic_2023.tsv",
}

_SUPPORTED_TASKS = []

_SOURCE_VERSION = "2023.0.0"

_SEACROWD_VERSION = "1.0.0"


class MalindoMorph(datasets.GeneratorBasedBuilder):
    """MALINDO Morph is a morphological dictionary for Malay (bahasa Melayu) and Indonesian (bahasa Indonesia) language. It provides morphological information (root, prefix, suffix, circumfix, reduplication) for over 200,000 surface forms."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "root": datasets.Value("string"),
                    "bentuk_jadian": datasets.Value("string"),
                    "prefix": datasets.Value("string"),
                    "suffix": datasets.Value("string"),
                    "circumfix": datasets.Value("string"),
                    "reduplication": datasets.Value("string"),
                    "source": datasets.Value("string"),
                    "stem": datasets.Value("string"),
                    "lemma": datasets.Value("string"),
                }
            )

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        urls = _URLS[_DATASETNAME]
        file = dl_manager.download_and_extract(urls)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": file,
                    "split": "train",
                },
            )
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        rows = []
        with open(filepath, encoding="utf8") as file:
            for line in file:
                row = line.split("\t")
                row[-1] = row[-1].split("\n")[0] # remove newlines from lemma feature
                rows.append(row)

        if self.config.schema == "source":
            for key, row in enumerate(rows):
                example = {"id": row[0], "root": row[1], "bentuk_jadian": row[2], "prefix": row[3], "suffix": row[4], "circumfix": row[5], "reduplication": row[6], "source": row[7], "stem": row[8], "lemma": row[9]}
                yield key, example



from pathlib import Path
from typing import List

import datasets
import json
import os

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks, DEFAULT_SOURCE_VIEW_NAME, DEFAULT_SEACROWD_VIEW_NAME

_DATASETNAME = "titml_idn"
_SOURCE_VIEW_NAME = DEFAULT_SOURCE_VIEW_NAME
_UNIFIED_VIEW_NAME = DEFAULT_SEACROWD_VIEW_NAME

_LANGUAGES = ["ind"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_LOCAL = False
_CITATION = """\
@inproceedings{lestari2006titmlidn,
  title={A large vocabulary continuous speech recognition system for Indonesian language},
  author={Lestari, Dessi Puji and Iwano, Koji and Furui, Sadaoki},
  booktitle={15th Indonesian Scientific Conference in Japan Proceedings},
  pages={17--22},
  year={2006}
}
"""

_DESCRIPTION = """\
TITML-IDN (Tokyo Institute of Technology Multilingual - Indonesian) is collected to build a pioneering Indonesian Large Vocabulary Continuous Speech Recognition (LVCSR) System. In order to build an LVCSR system, high accurate acoustic models and large-scale language models are essential. Since Indonesian speech corpus was not available yet, we tried to collect speech data from 20 Indonesian native speakers (11 males and 9 females) to construct a speech corpus for training the acoustic model based on Hidden Markov Models (HMMs). A text corpus which was collected by ILPS, Informatics Institute, University of Amsterdam, was used to build a 40K-vocabulary dictionary and a n-gram language model.
"""

_HOMEPAGE = "http://research.nii.ac.jp/src/en/TITML-IDN.html"

_LICENSE = "For research purposes only. If you use this corpus, you have to cite (Lestari et al, 2006)."

_URLs = {"titml-idn": "https://huggingface.co/datasets/holylovenia/TITML-IDN/resolve/main/IndoLVCSR.zip"}

_SUPPORTED_TASKS = [Tasks.SPEECH_RECOGNITION]

_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class TitmlIdn(datasets.GeneratorBasedBuilder):
    """TITML-IDN is a speech recognition dataset containing Indonesian speech collected with transcriptions from newpaper and magazine articles."""

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="titml_idn_source",
            version=datasets.Version(_SOURCE_VERSION),
            description="TITML-IDN source schema",
            schema="source",
            subset_id="titml_idn",
        ),
        SEACrowdConfig(
            name="titml_idn_seacrowd_sptext",
            version=datasets.Version(_SEACROWD_VERSION),
            description="TITML-IDN Nusantara schema",
            schema="seacrowd_sptext",
            subset_id="titml_idn",
        ),
    ]

    DEFAULT_CONFIG_NAME = "titml_idn_source"

    def _info(self):
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "speaker_id": datasets.Value("string"),
                    "path": datasets.Value("string"),
                    "audio": datasets.Audio(sampling_rate=16_000),
                    "text": datasets.Value("string"),
                }
            )
        elif self.config.schema == "seacrowd_sptext":
            features = schemas.speech_text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
            task_templates=[datasets.AutomaticSpeechRecognition(audio_column="audio", transcription_column="text")],
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        base_path = dl_manager.download_and_extract(_URLs["titml-idn"])

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": base_path},
            ),
        ]

    def _generate_examples(self, filepath: Path, n_speakers=20):

        if self.config.schema == "source" or self.config.schema == "seacrowd_sptext":

            for speaker_id in range(1, n_speakers + 1):
                speaker_id = str(speaker_id).zfill(2)
                dir_path = os.path.join(filepath, speaker_id)
                transcription_path = os.path.join(dir_path, "script~")

                with open(transcription_path, "r+") as f:
                    for line in f:
                        audio_id = line[2:8]
                        text = line[9:].strip()
                        wav_path = os.path.join(dir_path, "{}.wav".format(audio_id))

                        if os.path.exists(wav_path):
                            if self.config.schema == "source":
                                ex = {
                                    "id": audio_id,
                                    "speaker_id": speaker_id,
                                    "path": wav_path,
                                    "audio": wav_path,
                                    "text": text,
                                }
                                yield audio_id, ex
                            elif self.config.schema == "seacrowd_sptext":
                                ex = {
                                    "id": audio_id,
                                    "speaker_id": speaker_id,
                                    "path": wav_path,
                                    "audio": wav_path,
                                    "text": text,
                                    "metadata": {
                                        "speaker_age": None,
                                        "speaker_gender": None,
                                    }
                                }
                                yield audio_id, ex
        else:
            raise ValueError(f"Invalid config: {self.config.name}")



"""
General Video-to-Text Schema, including:
- Video captioning
- Video to text retrieval

Video datasets can be very large. For datasets with remote videos
('video_path' = video URL), ensure that the URL is publicly accessible
and the video is downloadable. Extra caution is needed, as the URL
might contain harmful and/or malicious files.
"""
import datasets

features = datasets.Features(
    {
        "id": datasets.Value("string"),
        "video_path": datasets.Value("string"),
        "text": datasets.Value("string"),
        "metadata": {
            "resolution": {
                "width": datasets.Value("int64"),
                "height": datasets.Value("int64"),
            },
            "duration": datasets.Value("float32"),
            "fps": datasets.Value("float32"),
        },
    }
)



from pathlib import Path

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """
@article{thoma2018wili,
  title={The WiLI benchmark dataset for written language identification},
  author={Thoma, Martin},
  journal={arXiv preprint arXiv:1801.07779},
  year={2018}
}
"""

_DATASETNAME = "wili_2018"

_DESCRIPTION = """
WiLI-2018 is a Wikipedia language identification benchmark dataset. It contains 235000 paragraphs from 235 languages.
The dataset is balanced, and a train-test split is provided.
"""

_HOMEPAGE = "https://zenodo.org/records/841984"

_LANGUAGES = ["nrm", "jav", "min", "lao", "mya", "pag", "ind", "cbk", "tet", "tha", "ceb", "tgl", "bjn", "bcl", "vie"]

_LICENSE = Licenses.ODBL.value

_LOCAL = False

_URLS = {
    _DATASETNAME: {"train": "https://drive.google.com/uc?export=download&id=1ZzlIQvw1KNBG97QQCfdatvVrrbeLaM1u", "test": "https://drive.google.com/uc?export=download&id=1Xx4kFc1Xdzz8AhDasxZ0cSa-a35EQSDZ"},
}

_SUPPORTED_TASKS = [Tasks.LANGUAGE_IDENTIFICATION]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


_CLASSES = [
    "cdo",
    "glk",
    "jam",
    "lug",
    "san",
    "rue",
    "wol",
    "new",
    "mwl",
    "bre",
    "ara",
    "hye",
    "xmf",
    "ext",
    "cor",
    "yor",
    "div",
    "asm",
    "lat",
    "cym",
    "hif",
    "ace",
    "kbd",
    "tgk",
    "rus",
    "nso",
    "mya",
    "msa",
    "ava",
    "cbk",
    "urd",
    "deu",
    "swa",
    "pus",
    "bxr",
    "udm",
    "csb",
    "yid",
    "vro",
    "por",
    "pdc",
    "eng",
    "tha",
    "hat",
    "lmo",
    "pag",
    "jav",
    "chv",
    "nan",
    "sco",
    "kat",
    "bho",
    "bos",
    "kok",
    "oss",
    "mri",
    "fry",
    "cat",
    "azb",
    "kin",
    "hin",
    "sna",
    "dan",
    "egl",
    "mkd",
    "ron",
    "bul",
    "hrv",
    "som",
    "pam",
    "nav",
    "ksh",
    "nci",
    "khm",
    "sgs",
    "srn",
    "bar",
    "cos",
    "ckb",
    "pfl",
    "arz",
    "roa-tara",
    "fra",
    "mai",
    "zh-yue",
    "guj",
    "fin",
    "kir",
    "vol",
    "hau",
    "afr",
    "uig",
    "lao",
    "swe",
    "slv",
    "kor",
    "szl",
    "srp",
    "dty",
    "nrm",
    "dsb",
    "ind",
    "wln",
    "pnb",
    "ukr",
    "bpy",
    "vie",
    "tur",
    "aym",
    "lit",
    "zea",
    "pol",
    "est",
    "scn",
    "vls",
    "stq",
    "gag",
    "grn",
    "kaz",
    "ben",
    "pcd",
    "bjn",
    "krc",
    "amh",
    "diq",
    "ltz",
    "ita",
    "kab",
    "bel",
    "ang",
    "mhr",
    "che",
    "koi",
    "glv",
    "ido",
    "fao",
    "bak",
    "isl",
    "bcl",
    "tet",
    "jpn",
    "kur",
    "map-bms",
    "tyv",
    "olo",
    "arg",
    "ori",
    "lim",
    "tel",
    "lin",
    "roh",
    "sqi",
    "xho",
    "mlg",
    "fas",
    "hbs",
    "tam",
    "aze",
    "lad",
    "nob",
    "sin",
    "gla",
    "nap",
    "snd",
    "ast",
    "mal",
    "mdf",
    "tsn",
    "nds",
    "tgl",
    "nno",
    "sun",
    "lzh",
    "jbo",
    "crh",
    "pap",
    "oci",
    "hak",
    "uzb",
    "zho",
    "hsb",
    "sme",
    "mlt",
    "vep",
    "lez",
    "nld",
    "nds-nl",
    "mrj",
    "spa",
    "ceb",
    "ina",
    "heb",
    "hun",
    "que",
    "kaa",
    "mar",
    "vec",
    "frp",
    "ell",
    "sah",
    "eus",
    "ces",
    "slk",
    "chr",
    "lij",
    "nep",
    "srd",
    "ilo",
    "be-tarask",
    "bod",
    "orm",
    "war",
    "glg",
    "mon",
    "gle",
    "min",
    "ibo",
    "ile",
    "epo",
    "lav",
    "lrc",
    "als",
    "mzn",
    "rup",
    "fur",
    "tat",
    "myv",
    "pan",
    "ton",
    "kom",
    "wuu",
    "tcy",
    "tuk",
    "kan",
    "ltg",
]


class Wili2018Dataset(datasets.GeneratorBasedBuilder):
    """A benchmark dataset for language identification and contains 235000 paragraphs of 235 languages."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=_DATASETNAME,
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_text",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema="seacrowd_text",
            subset_id=_DATASETNAME,
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "sentence": datasets.Value("string"),
                    "label": datasets.ClassLabel(names=_CLASSES),
                }
            )

        elif self.config.schema == "seacrowd_text":
            features = schemas.text_features(_CLASSES)

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> list[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        urls = _URLS[_DATASETNAME]
        data_dir = dl_manager.download_and_extract(urls)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": data_dir, "split": "train"},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": data_dir, "split": "test"},
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> tuple[int, dict]:
        if self.config.schema == "source":
            with open(filepath[split], encoding="utf-8") as f:
                for i, line in enumerate(f):
                    text, label = line.rsplit(",", 1)
                    text = text.strip('"')
                    label = int(label.strip())
                    yield i, {"sentence": text, "label": _CLASSES[label - 1]}

        elif self.config.schema == "seacrowd_text":
            with open(filepath[split], encoding="utf-8") as f:
                for i, line in enumerate(f):
                    text, label = line.rsplit(",", 1)
                    text = text.strip('"')
                    label = int(label.strip())
                    yield i, {"id": str(i), "text": text, "label": _CLASSES[label - 1]}



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from itertools import chain
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks

_CITATION = """\
@inproceedings{wibowo-etal-2021-indocollex,
    title = "{I}ndo{C}ollex: A Testbed for Morphological Transformation of {I}ndonesian Word Colloquialism",
    author = {Wibowo, Haryo Akbarianto  and Nityasya, Made Nindyatama  and Aky{\"u}rek, Afra Feyza  and Fitriany, Suci  and Aji, Alham Fikri  and Prasojo, Radityo Eko  and Wijaya, Derry Tanti},
    booktitle = "Findings of the Association for Computational Linguistics: ACL-IJCNLP 2021",
    month = aug,
    year = "2021",
    address = "Online",
    publisher = "Association for Computational Linguistics",
    url = "https://aclanthology.org/2021.findings-acl.280",
    doi = "10.18653/v1/2021.findings-acl.280",
    pages = "3170--3183",
}"""

_LANGUAGES = ["ind"]
_LOCAL = False

_DATASETNAME = "indocollex"

_DESCRIPTION = """\
IndoCollex: A Testbed for Morphological Transformation of Indonesian Colloquial Words
"""

_HOMEPAGE = "https://github.com/haryoa/indo-collex"

_LICENSE = "CC BY-SA 4.0"

_URLS = {
    _DATASETNAME: {
        "train": "https://github.com/haryoa/indo-collex/raw/main/data/full.csv",
    },
    f"{_DATASETNAME}_f2i": {
        "train": "https://github.com/haryoa/indo-collex/raw/main/data/formal_to_informal/train.csv",
        "dev": "https://github.com/haryoa/indo-collex/raw/main/data/formal_to_informal/dev.csv",
        "test": "https://github.com/haryoa/indo-collex/raw/main/data/formal_to_informal/test.csv",
    },
    f"{_DATASETNAME}_i2f": {
        "train": "https://github.com/haryoa/indo-collex/raw/main/data/informal_to_formal/train.csv",
        "dev": "https://github.com/haryoa/indo-collex/raw/main/data/informal_to_formal/dev.csv",
        "test": "https://github.com/haryoa/indo-collex/raw/main/data/informal_to_formal/test.csv",
    },
}

_SUPPORTED_TASKS = [Tasks.MORPHOLOGICAL_INFLECTION]

_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class NewDataset(datasets.GeneratorBasedBuilder):
    """IndoCollex: A Testbed for Morphological Transformation of Indonesian Colloquial Words"""

    label_classes = ["acronym", "affixation", "disemvoweling", "rev", "shorten", "sound-alter", "space-dash"]

    BUILDER_CONFIGS = list(
        chain(
            *[
                [
                    SEACrowdConfig(
                        name=f"{_DATASETNAME}{suffix}_source",
                        version=datasets.Version(_SOURCE_VERSION),
                        description=f"{_DATASETNAME} source schema",
                        schema="source",
                        subset_id=f"{_DATASETNAME}{suffix}",
                    ),
                    SEACrowdConfig(
                        name=f"{_DATASETNAME}{suffix}_seacrowd_pairs_multi",
                        version=datasets.Version(_SEACROWD_VERSION),
                        description=f"{_DATASETNAME} Nusantara schema",
                        schema="seacrowd_pairs_multi",
                        subset_id=f"{_DATASETNAME}{suffix}",
                    ),
                ]
                for suffix in ["", "_f2i", "_i2f"]
            ]
        )
    )

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "no": datasets.Value("string"),
                    "transformed": datasets.Value("string"),
                    "original-for": datasets.Value("string"),
                    "transformation": datasets.Value("string"),
                }
            )

        elif self.config.schema == "seacrowd_pairs_multi":
            features = schemas.pairs_multi_features(self.label_classes)

        else:
            raise NotImplementedError(f"Schema '{self.config.schema}' is not defined.")

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""

        urls = _URLS[self.config.subset_id]
        data_paths = dl_manager.download(urls)

        ret = [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": data_paths["train"]},
            )
        ]

        if len(data_paths) > 1:
            ret.extend(
                [
                    datasets.SplitGenerator(
                        name=datasets.Split.TEST,
                        gen_kwargs={"filepath": data_paths["test"]},
                    ),
                    datasets.SplitGenerator(
                        name=datasets.Split.VALIDATION,
                        gen_kwargs={"filepath": data_paths["dev"]},
                    ),
                ]
            )

        return ret

    def _generate_examples(self, filepath: Path) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""

        with open(filepath, "r", encoding="utf8") as f:
            dataset = list(map(lambda l: l.rstrip("\r\n").split(","), f))

        _assert = set(map(len, dataset))
        if _assert != {4}:
            raise AssertionError(f"Expecting exactly 4 fields (no, transformed, base, label), but found: {_assert}")

        _assert = dataset[0]
        source_columns = ["no", "transformed", "original-for", "transformation"]
        if _assert != source_columns:
            raise AssertionError(f"The expected header is not found. {_assert}")

        dataset = dataset[1:]

        if self.config.schema == "source":
            for key, ex in enumerate(dataset):
                yield key, dict(zip(source_columns, ex))

        elif self.config.schema == "seacrowd_pairs_multi":
            for key, ex in enumerate(dataset):
                yield key, {
                    "id": str(key),
                    "text_1": ex[2],
                    "text_2": ex[1],
                    "label": [ex[3]],
                }



from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks
from seacrowd.utils import schemas

import pandas as pd


_CITATION = """\
@INPROCEEDINGS{8629151,
author={Aliyah Salsabila, Nikmatun and Ardhito Winatmoko, Yosef and Akbar Septiandri, Ali and Jamal, Ade},
booktitle={2018 International Conference on Asian Language Processing (IALP)},
title={Colloquial Indonesian Lexicon},
year={2018},
volume={},
number={},
pages={226-229},
doi={10.1109/IALP.2018.8629151}}
"""

_LANGUAGES = ["ind"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_LOCAL = False

_DATASETNAME = "kamus_alay"

_DESCRIPTION = """\
Kamus Alay provide a lexicon for text normalization of Indonesian colloquial words.
It contains 3,592 unique colloquial words-also known as “bahasa alay” -and manually annotated them
with the normalized form. We built this lexicon from Instagram comments provided by Septiandri & Wibisono (2017)
"""

_HOMEPAGE = "https://ieeexplore.ieee.org/abstract/document/8629151"

_LICENSE = "Unknown"

_URLS = {
    _DATASETNAME: "https://raw.githubusercontent.com/nasalsabila/kamus-alay/master/colloquial-indonesian-lexicon.csv",
}

_SUPPORTED_TASKS = [Tasks.MORPHOLOGICAL_INFLECTION]

# Dataset does not have versioning
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class KamusAlay(datasets.GeneratorBasedBuilder):
    """Kamus Alay is a dataset of lexicon for text normalization of Indonesian colloquial word"""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    label_classes = [
        "abreviasi",
        "afiksasi",
        "akronim",
        "anaptiksis",
        "coinage",
        "elongasi",
        "homofon",
        "metatesis",
        "modifikasi vokal",
        "monoftongisasi",
        "naturalisasi",
        "pungtuasi",
        "reduplikasi",
        "salah ketik",
        "subtitusi",
        "word-value letter",
        "zeroisasi",
    ]

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="kamus_alay_source",
            version=SOURCE_VERSION,
            description="Kamus Alay source schema",
            schema="source",
            subset_id="kamus_alay",
        ),
        SEACrowdConfig(
            name="kamus_alay_seacrowd_pairs_multi",
            version=SEACROWD_VERSION,
            description="Kamus Alay Nusantara schema",
            schema="seacrowd_pairs_multi",
            subset_id="kamus_alay",
        ),
    ]

    DEFAULT_CONFIG_NAME = "kamus_alay_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "slang": datasets.Value("string"),
                    "formal": datasets.Value("string"),
                    "in_dictionary": datasets.Value("bool"),
                    "context": datasets.Value("string"),
                    "categories": datasets.Sequence(datasets.Value("string")),
                }
            )
        elif self.config.schema == "seacrowd_pairs_multi":
            features = schemas.pairs_multi_features(self.label_classes)

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        urls = _URLS[_DATASETNAME]

        data_dir = Path(dl_manager.download(urls))
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_dir,
                    "split": "train",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        # Dataset does not have id, using row index as id
        df = pd.read_csv(filepath, encoding="ISO-8859-1").reset_index()
        df.columns = ["id", "slang", "formal", "is_in_dictionary", "example", "category1", "category2", "category3"]

        if self.config.schema == "source":
            for row in df.itertuples():
                ex = {
                    "slang": row.slang,
                    "formal": row.formal,
                    "in_dictionary": row.is_in_dictionary,
                    "context": row.example,
                    "categories": [c for c in (row.category1, row.category2, row.category3) if c != "0"],
                }
                yield row.id, ex

        elif self.config.schema == "seacrowd_pairs_multi":
            for row in df.itertuples():
                ex = {
                    "id": str(row.id),
                    "text_1": row.formal,
                    "text_2": row.slang,
                    "label": [c for c in (row.category1, row.category2, row.category3) if c != "0"],
                }
                yield row.id, ex
        else:
            raise ValueError(f"Invalid config: {self.config.name}")



from pathlib import Path
from typing import List

import datasets
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import DEFAULT_SEACROWD_VIEW_NAME, DEFAULT_SOURCE_VIEW_NAME, Tasks

_DATASETNAME = "emotion_id_opinion"
_SOURCE_VIEW_NAME = DEFAULT_SOURCE_VIEW_NAME
_UNIFIED_VIEW_NAME = DEFAULT_SEACROWD_VIEW_NAME

_LANGUAGES = ["ind"]  # We follow ISO639-3 langauge code (https://iso639-3.sil.org/code_tables/639/data)
_LOCAL = False
_CITATION = """\
@article{RICCOSAN2022108465,
title = {Emotion dataset from Indonesian public opinion},
journal = {Data in Brief},
volume = {43},
pages = {108465},
year = {2022},
issn = {2352-3409},
doi = {https://doi.org/10.1016/j.dib.2022.108465},
url = {https://www.sciencedirect.com/science/article/pii/S2352340922006588},
author = { Riccosan and Karen Etania Saputra and Galih Dea Pratama and Andry Chowanda},
keywords = {Emotion classification, Dataset, Tweet, Indonesia},
abstract = {An opinion is a type of judgment or a person's point of view about something. Twitter is a popular social media platform that includes a lot of public opinions and would be a suitable location to mine data in text form. With its vast population and active Twitter user base, Indonesia has the potential to be a source of opinion data mining. An opinion may be processed and result in the form of a person's emotional response towards something, such as whether they like, hate, love, or are happy about it. Upon that basis, a dataset of Indonesian-language tweets conveying public opinion on various topics was formed. The fact that there are only limited publicly available emotions text datasets in the Indonesian language supports our basis in this research to form our emotion dataset. The gathered data was cleaned and normalized in the pre-processing stage to the necessary form for study on the task of classifying emotions in Indonesian. The data collected is annotated with six emotional labels: anger, fear, joy, love, sad, and neutral.}
}
"""

_DESCRIPTION = """\
Emotion ID Opinion is a dataset of Indonesian-language tweets conveying public opinion on a variety of topics.
It comtains 7080 indunesian tweets and a person's emotion response towards each tweet.
The data is annotated with six emotional labels, namely anger, fear, joy, love, sad, and neutral.
"""

_HOMEPAGE = "https://github.com/Ricco48/Emotion-Dataset-from-Indonesian-Public-Opinion"

_LICENSE = "Creative Commons Attribution Share-Alike 4.0 International"

_URLs = {
    "anger": "https://raw.githubusercontent.com/Ricco48/Emotion-Dataset-from-Indonesian-Public-Opinion/main/Emotion%20Dataset%20from%20Indonesian%20Public%20Opinion/AngerData.csv",
    "fear": "https://raw.githubusercontent.com/Ricco48/Emotion-Dataset-from-Indonesian-Public-Opinion/main/Emotion%20Dataset%20from%20Indonesian%20Public%20Opinion/FearData.csv",
    "joy": "https://raw.githubusercontent.com/Ricco48/Emotion-Dataset-from-Indonesian-Public-Opinion/main/Emotion%20Dataset%20from%20Indonesian%20Public%20Opinion/JoyData.csv",
    "love": "https://raw.githubusercontent.com/Ricco48/Emotion-Dataset-from-Indonesian-Public-Opinion/main/Emotion%20Dataset%20from%20Indonesian%20Public%20Opinion/LoveData.csv",
    "sad": "https://raw.githubusercontent.com/Ricco48/Emotion-Dataset-from-Indonesian-Public-Opinion/main/Emotion%20Dataset%20from%20Indonesian%20Public%20Opinion/SadData.csv",
    "neutral": "https://raw.githubusercontent.com/Ricco48/Emotion-Dataset-from-Indonesian-Public-Opinion/main/Emotion%20Dataset%20from%20Indonesian%20Public%20Opinion/NeutralData.csv"
}

_SUPPORTED_TASKS = [Tasks.EMOTION_CLASSIFICATION]

_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class EmoIdOpinion(datasets.GeneratorBasedBuilder):

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="emotion_id_opinion_source",
            version=datasets.Version(_SOURCE_VERSION),
            description="EmoIdOpinion source schema",
            schema="source",
            subset_id="emotion_id_opinion",
        ),
        SEACrowdConfig(
            name="emotion_id_opinion_seacrowd_text",
            version=datasets.Version(_SEACROWD_VERSION),
            description="EmoIdOpinion Nusantara schema",
            schema="seacrowd_text",
            subset_id="emotion_id_opinion",
        ),
    ]

    DEFAULT_CONFIG_NAME = "emotion_id_opinion_source"

    def _info(self):
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "index": datasets.Value("string"),
                    "tweet": datasets.Value("string"),
                    "label": datasets.Value("string"),
                }
            )
        elif self.config.schema == "seacrowd_text":
            features = schemas.text_features(["Joy", "Love", "Fear", "Anger", "Sad", "Neutral"])

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        anger_tsv_path = Path(dl_manager.download_and_extract(_URLs["anger"]))
        fear_tsv_path = Path(dl_manager.download_and_extract(_URLs["fear"]))
        joy_tsv_path = Path(dl_manager.download_and_extract(_URLs["joy"]))
        love_tsv_path = Path(dl_manager.download_and_extract(_URLs["love"]))
        neutral_tsv_path = Path(dl_manager.download_and_extract(_URLs["neutral"]))
        sad_tsv_path = Path(dl_manager.download_and_extract(_URLs["sad"]))

        data_files = {
            "anger": anger_tsv_path,
            "fear": fear_tsv_path,
            "joy": joy_tsv_path,
            "love": love_tsv_path,
            "neutral": neutral_tsv_path,
            "sad": sad_tsv_path
        }

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": [
                    data_files["anger"],
                    data_files["fear"],
                    data_files["joy"],
                    data_files["love"],
                    data_files["neutral"],
                    data_files["sad"]
                ]},
            ),
        ]

    def _generate_examples(self, filepath: List[Path]):
        increment = 0
        for i, fp in enumerate(filepath):
        # df = pd.concat([pd.read_csv(fp, sep="\t", header="infer").reset_index() for fp in filepath])
            df = pd.read_csv(fp, sep="\t", header="infer").reset_index()
            df.columns = ["id", "Tweet", "Label"]

            if self.config.schema == "source":
                for row in df.itertuples():
                    ex = {"index": str(increment + row.id), "tweet": row.Tweet, "label": row.Label}
                    yield increment + row.id, ex
            elif self.config.schema == "seacrowd_text":
                for row in df.itertuples():
                    ex = {"id": str(increment + row.id), "text": row.Tweet, "label": row.Label}
                    yield increment + row.id, ex
            else:
                raise ValueError(f"Invalid config: {self.config.name}")

            increment += row.id + 1



from pathlib import Path
from typing import Dict, List, Tuple

import datasets
from datasets.download.download_manager import DownloadManager

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """
@inproceedings{flores-radev-2022-look,
    title = "Look Ma, Only 400 Samples! Revisiting the Effectiveness of Automatic N-Gram Rule Generation for Spelling Normalization in {F}ilipino",
    author = "Flores, Lorenzo Jaime  and
      Radev, Dragomir",
    booktitle = "Proceedings of The Third Workshop on Simple and Efficient Natural Language Processing (SustaiNLP)",
    month = dec,
    year = "2022",
    address = "Abu Dhabi, United Arab Emirates (Hybrid)",
    publisher = "Association for Computational Linguistics",
    url = "https://aclanthology.org/2022.sustainlp-1.5",
    pages = "29--35",
}
"""

_LOCAL = False
_LANGUAGES = ["fil"]
_DATASETNAME = "filipino_slang_norm"
_DESCRIPTION = """\
This dataset contains 398 abbreviated and/or contracted Filipino words used in
Facebook comments made on weather advisories from a Philippine weather bureau.
volunteers.
"""

_HOMEPAGE = "https://github.com/ljyflores/efficient-spelling-normalization-filipino"
_LICENSE = Licenses.UNKNOWN.value
_URLS = {
    "train": "https://github.com/ljyflores/efficient-spelling-normalization-filipino/raw/main/data/train_words.csv",
    "test": "https://github.com/ljyflores/efficient-spelling-normalization-filipino/raw/main/data/test_words.csv",
}

_SUPPORTED_TASKS = [Tasks.MULTILEXNORM]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class FilipinoSlangNormDataset(datasets.GeneratorBasedBuilder):
    """Filipino Slang Norm dataset by Flores and Radev (2022)"""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    SEACROWD_SCHEMA_NAME = "t2t"

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=_DATASETNAME,
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_{SEACROWD_SCHEMA_NAME}",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}",
            subset_id=_DATASETNAME,
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "src_sent": datasets.Value("string"),
                    "norm_sent": datasets.Value("string"),
                }
            )
        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            features = schemas.text2text_features
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        data_files = {
            "train": Path(dl_manager.download_and_extract(_URLS["train"])),
            "test": Path(dl_manager.download_and_extract(_URLS["test"])),
        }

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_files["train"],
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": data_files["test"],
                    "split": "test",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yield examples as (key, example) tuples"""
        with open(filepath, encoding="utf-8") as f:
            for guid, line in enumerate(f):
                src_sent, norm_sent = line.strip("\n").split(",")
                if self.config.schema == "source":
                    example = {
                        "id": str(guid),
                        "src_sent": src_sent,
                        "norm_sent": norm_sent,
                    }
                elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
                    example = {
                        "id": str(guid),
                        "text_1": src_sent,
                        "text_2": norm_sent,
                        "text_1_name": "src_sent",
                        "text_2_name": "norm_sent",
                    }
                yield guid, example



import ast
from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import pandas as pd

from seacrowd.sea_datasets.facqa.utils.facqa_utils import (getAnswerString, listToString)
from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks

_CITATION = """
@inproceedings{purwarianti2007machine,
  title={A Machine Learning Approach for Indonesian Question Answering System},
  author={Ayu Purwarianti, Masatoshi Tsuchiya, and Seiichi Nakagawa},
  booktitle={Proceedings of Artificial Intelligence and Applications },
  pages={573--578},
  year={2007}
}
"""

_LANGUAGES = ["ind"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_LOCAL = False

_DATASETNAME = "facqa"

_DESCRIPTION = """
FacQA: The goal of the FacQA dataset is to find the answer to a question from a provided short passage from a news article.
Each row in the FacQA dataset consists of a question, a short passage, and a label phrase, which can be found inside the
corresponding short passage. There are six categories of questions: date, location, name,
organization, person, and quantitative.
"""

_HOMEPAGE = "https://github.com/IndoNLP/indonlu"

_LICENSE = "CC-BY-SA 4.0"

_URLS = {
    _DATASETNAME: {
        "test": "https://raw.githubusercontent.com/IndoNLP/indonlu/master/dataset/facqa_qa-factoid-itb/test_preprocess.csv",
        "train": "https://raw.githubusercontent.com/IndoNLP/indonlu/master/dataset/facqa_qa-factoid-itb/train_preprocess.csv",
        "validation": "https://raw.githubusercontent.com/IndoNLP/indonlu/master/dataset/facqa_qa-factoid-itb/valid_preprocess.csv",
    }
}

_SUPPORTED_TASKS = [Tasks.QUESTION_ANSWERING]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class FacqaDataset(datasets.GeneratorBasedBuilder):
    """FacQA dataset is a labeled dataset for indonesian question answering task"""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="facqa_source",
            version=SOURCE_VERSION,
            description="FacQA source schema",
            schema="source",
            subset_id="facqa",
        ),
        SEACrowdConfig(
            name="facqa_seacrowd_qa",
            version=SEACROWD_VERSION,
            description="FacQA Nusantara schema",
            schema="seacrowd_qa",
            subset_id="facqa",
        ),
    ]

    DEFAULT_CONFIG_NAME = "facqa_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "index": datasets.Value("int64"),
                    "question": [datasets.Value("string")],
                    "passage": [datasets.Value("string")],
                    "seq_label": [datasets.Value("string")],
                }
            )
        elif self.config.schema == "seacrowd_qa":
            features = schemas.qa_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        urls = _URLS[_DATASETNAME]
        train_csv_path = Path(dl_manager.download_and_extract(urls["train"]))
        validation_csv_path = Path(dl_manager.download_and_extract(urls["validation"]))
        test_csv_path = Path(dl_manager.download_and_extract(urls["test"]))
        data_files = {
            "train": train_csv_path,
            "validation": validation_csv_path,
            "test": test_csv_path,
        }
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_files["train"],
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": data_files["test"],
                    "split": "test",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": data_files["validation"],
                    "split": "dev",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        df = pd.read_csv(filepath, sep=",", header="infer").reset_index()
        if self.config.schema == "source":
            for row in df.itertuples():
                entry = {"index": row.index, "question": ast.literal_eval(row.question), "passage": ast.literal_eval(row.passage), "seq_label": ast.literal_eval(row.seq_label)}
                yield row.index, entry

        elif self.config.schema == "seacrowd_qa":
            for row in df.itertuples():
                entry = {
                    "id": str(row.index),
                    "question_id": str(row.index),
                    "document_id": str(row.index),
                    "question": listToString(ast.literal_eval(row.question)),
                    "type": "extractive",
                    "choices": [],
                    "context": listToString(ast.literal_eval(row.passage)),
                    "answer": [getAnswerString(ast.literal_eval(row.passage), ast.literal_eval(row.seq_label))],
                    "meta": {}
                }
                yield row.index, entry



from typing import Iterable, Optional

import pandas as pd
from conllu import parse


def load_conll_data(file_path):
    # Read file
    data = open(file_path, "r", encoding="utf8").readlines()

    # Prepare buffer
    dataset = []
    sentence, seq_label = [], []
    for line in data:
        if len(line.strip()) > 0:
            token, label = line[:-1].split("\t")
            sentence.append(token)
            seq_label.append(label)
        else:
            dataset.append({"sentence": sentence, "label": seq_label})
            sentence = []
            seq_label = []
    return dataset


def load_ud_data(filepath, filter_kwargs=None, assert_fn=None):
    """
    Load and parse conllu data.

    Proposed by @fhudi for issue #34 and #9.

    :param filepath: file path
    :param filter_kwargs: filtering tokens, see conllu.models.TokenList.filter()
    :param assert_fn: assertion to make sure raw data is in the expected format
    :return: generator with schema following CONLLU
    """
    dataset_raw = parse(open(filepath).read())

    filter_kwargs = filter_kwargs or dict()
    if callable(assert_fn):
        for token_list in dataset_raw:
            assert_fn(token_list)

    return map(lambda sent: {**sent.metadata, **pd.DataFrame(sent.filter(**filter_kwargs)).to_dict(orient="list")}, dataset_raw)


def load_ud_data_as_seacrowd_kb(filepath, dataset_source: Iterable = tuple(), morph_exceptions: Optional[Iterable] = None):
    """
    Load and parse conllu data, followed by mapping its elements to Nusantara Knowledge Base schema.

    Proposed by @fhudi for issue #34 and #9.

    :param filepath: file path
    :param dataset_source: dataset with source schema (output of load_ud_data())
    :param morph_exceptions: list of surface and morphological-form pairs
    :return: generator for Nusantara KB schema
    """
    dataset_source = dataset_source or list(load_ud_data(filepath))

    def as_nusa_kb(tokens):
        sent_id = tokens["sent_id"]
        offsets = get_span_offsets(tokens["form"], tokens["text"], morph_exceptions=(morph_exceptions or []))
        return {
            "id": sent_id,
            "passages": [
                {
                    "id": f"{sent_id}_passages",
                    "type": "",
                    "text": [tokens["text"]],
                    "offsets": [(0, len(tokens["text"]))],
                }
            ],
            "entities": [
                {
                    "id": f"{sent_id}_EntID_{ent_id}",
                    "type": ent_type,
                    "text": [ent_text],
                    "offsets": [offset],
                    "normalized": [
                        {
                            "db_name": norm_text,
                            "db_id": None,
                        }
                    ],
                }
                for (ent_id, ent_type, ent_text, offset, norm_text) in zip(tokens["id"], tokens["upos"], tokens["form"], offsets, tokens["lemma"])
            ],
            "relations": [
                {
                    "id": f"{sent_id}_RelID_{rel_id}",
                    "type": rel_type,
                    "arg1_id": f"{sent_id}_EntID_{rel_child}",
                    "arg2_id": f"{sent_id}_EntID_{rel_parent}",
                    "normalized": [],
                }
                for rel_id, (rel_type, rel_child, rel_parent) in enumerate(zip(tokens["deprel"], tokens["id"], tokens["head"]))
            ],
            "events": [],
            "coreferences": [],
        }

    return map(as_nusa_kb, dataset_source)


def get_span_offsets(spans_inorder, text_concatenated, delimiters={" "}, morph_exceptions=[]):
    """
    Getting the offset of each span assuming spans_inorder is retrieved by splitting text_concatenated using one of delimiters.

    :param spans_inorder: Iterable<String>
    :param text_concatenated: String
    :param delimiters: Set<char>
    :param morph_exceptions: Iterable<Tuple<String, String>>
    :return: List of pair (lo, hi) indicating the start index (inclusive) and end index (exclusive) of original text, respectively.
    """
    offsets = []
    span_idx, span = None, None

    def iter_char():
        nonlocal span_idx, span
        for span_idx, span in enumerate(spans_inorder):
            for st, ch in enumerate(span):
                yield len(span) if st == 0 else None, ch

    def is_morph_excepted():
        nonlocal span, text_concatenated, offset
        for surface, morph in morph_exceptions:
            if text_concatenated[offset : offset + len(surface)] == surface and span == morph:
                return True
        return False

    try:
        iterchar = iter(iter_char())
        span_len, cur_char = next(iterchar)
        for offset, j in enumerate(text_concatenated):
            if cur_char.lower() != j.lower():
                if j in delimiters or is_morph_excepted():
                    continue
                else:
                    err_msg = f"Char '{j}' at pos {offset} does not match char '{cur_char}' from" f" span #{span_idx} ('{span}'), and is not in delimiters {delimiters};"
                    raise AssertionError(err_msg)
            else:
                if span_len is not None:
                    offsets.append((offset, offset + span_len))
                span_len, cur_char = next(iterchar)
        raise AssertionError("Text is too short, not enough character for whole spans.")

    except StopIteration:
        return offsets



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import re
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@inproceedings{kjartansson18_sltu,
  author={Oddur Kjartansson and Supheakmungkol Sarin and Knot Pipatsrisawat and Martin Jansche and Linne Ha},
  title={{Crowd-Sourced Speech Corpora for Javanese, Sundanese, Sinhala, Nepali, and Bangladeshi Bengali}},
  year=2018,
  booktitle={Proc. 6th Workshop on Spoken Language Technologies for Under-Resourced Languages (SLTU 2018)},
  pages={52--55},
  doi={10.21437/SLTU.2018-11}
}
"""

_DATASETNAME = "openslr"

_DESCRIPTION = """\
This data set contains transcribed high-quality audio of Javanese, Sundanese, Burmese, Khmer. This data set\
come from 3 different projects under OpenSLR initiative
"""

_HOMEPAGE = "https://www.openslr.org/resources.php"

_LANGUAGES = ["mya", "jav", "sun", "khm"]

_LICENSE = Licenses.CC_BY_SA_4_0.value

_LOCAL = False

_RESOURCES = {
    "SLR35": {
        "language": "jav",
        "files": [
            "asr_javanese_0.zip",
            "asr_javanese_1.zip",
            "asr_javanese_2.zip",
            "asr_javanese_3.zip",
            "asr_javanese_4.zip",
            "asr_javanese_5.zip",
            "asr_javanese_6.zip",
            "asr_javanese_7.zip",
            "asr_javanese_8.zip",
            "asr_javanese_9.zip",
            "asr_javanese_a.zip",
            "asr_javanese_b.zip",
            "asr_javanese_c.zip",
            "asr_javanese_d.zip",
            "asr_javanese_e.zip",
            "asr_javanese_f.zip",
        ],
        "index_files": ["asr_javanese/utt_spk_text.tsv"] * 16,
        "data_dirs": ["asr_javanese/data"] * 16,
    },
    "SLR36": {
        "language": "sun",
        "files": [
            "asr_sundanese_0.zip",
            "asr_sundanese_1.zip",
            "asr_sundanese_2.zip",
            "asr_sundanese_3.zip",
            "asr_sundanese_4.zip",
            "asr_sundanese_5.zip",
            "asr_sundanese_6.zip",
            "asr_sundanese_7.zip",
            "asr_sundanese_8.zip",
            "asr_sundanese_9.zip",
            "asr_sundanese_a.zip",
            "asr_sundanese_b.zip",
            "asr_sundanese_c.zip",
            "asr_sundanese_d.zip",
            "asr_sundanese_e.zip",
            "asr_sundanese_f.zip",
        ],
        "index_files": ["asr_sundanese/utt_spk_text.tsv"] * 16,
        "data_dirs": ["asr_sundanese/data"] * 16,
    },
    "SLR41": {
        "language": "jav",
        "files": ["jv_id_female.zip", "jv_id_male.zip"],
        "index_files": ["jv_id_female/line_index.tsv", "jv_id_male/line_index.tsv"],
        "data_dirs": ["jv_id_female/wavs", "jv_id_male/wavs"],
    },
    "SLR42": {
        "language": "khm",
        "files": ["km_kh_male.zip"],
        "index_files": ["km_kh_male/line_index.tsv"],
        "data_dirs": ["km_kh_male/wavs"],
    },
    "SLR44": {
        "language": "sun",
        "files": ["su_id_female.zip", "su_id_male.zip"],
        "index_files": ["su_id_female/line_index.tsv", "su_id_male/line_index.tsv"],
        "data_dirs": ["su_id_female/wavs", "su_id_male/wavs"],
    },
    "SLR80": {
        "language": "mya",
        "files": ["my_mm_female.zip"],
        "index_files": ["line_index.tsv"],
        "data_dirs": [""],
    },
}
_URLS = {_DATASETNAME: "https://openslr.org/resources/{subset}"}

_SUPPORTED_TASKS = [Tasks.SPEECH_RECOGNITION]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class OpenSLRDataset(datasets.GeneratorBasedBuilder):
    """This data set contains transcribed high-quality audio of Javanese, Sundanese, Burmese, Khmer. This data set
    come from 3 different projects under OpenSLR initiative"""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(name=f"{_DATASETNAME}_{subset}_{_RESOURCES[subset]['language']}_source", version=datasets.Version(_SOURCE_VERSION), description=f"{_DATASETNAME} source schema", schema="source", subset_id=f"{_DATASETNAME}")
        for subset in _RESOURCES.keys()
    ] + [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_{subset}_{_RESOURCES[subset]['language']}_seacrowd_sptext", version=datasets.Version(_SEACROWD_VERSION), description=f"{_DATASETNAME} SEACrowd schema", schema="seacrowd_sptext", subset_id=f"{_DATASETNAME}"
        )
        for subset in _RESOURCES.keys()
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_SLR41_jav_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "path": datasets.Value("string"),
                    "audio": datasets.Audio(sampling_rate=48_000),
                    "sentence": datasets.Value("string"),
                }
            )
        elif self.config.schema == "seacrowd_sptext":
            features = schemas.speech_text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        subset = self.config.name.split("_")[1]
        urls = [f"{_URLS[_DATASETNAME].format(subset=subset[3:])}/{file}" for file in _RESOURCES[subset]["files"]]
        data_dir = dl_manager.download_and_extract(urls)

        path_to_indexs = [os.path.join(path, f"{_RESOURCES[subset]['index_files'][i]}") for i, path in enumerate(data_dir)]
        path_to_datas = [os.path.join(path, f"{_RESOURCES[subset]['data_dirs'][i]}") for i, path in enumerate(data_dir)]

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": [path_to_indexs, path_to_datas],
                    "split": "train",
                },
            )
        ]

    def _generate_examples(self, filepath, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        subset = self.config.name.split("_")[1]
        path_to_indexs, path_to_datas = filepath[0], filepath[1]
        counter = -1
        if subset in ["SLR35", "SLR36"]:
            sentence_index = {}
            for i, path_to_index in enumerate(path_to_indexs):
                with open(path_to_index, encoding="utf-8") as f:
                    lines = f.readlines()
                    for id_, line in enumerate(lines):
                        field_values = re.split(r"\t\t?", line.strip())
                        filename, user_id, sentence = field_values
                        sentence_index[filename] = sentence
                for path_to_data in sorted(Path(path_to_datas[i]).rglob("*.flac")):
                    filename = path_to_data.stem
                    if path_to_data.stem not in sentence_index:
                        continue
                    path = str(path_to_data.resolve())
                    sentence = sentence_index[filename]
                    counter += 1
                    if self.config.schema == "source":
                        example = {"path": path, "audio": path, "sentence": sentence}
                    elif self.config.schema == "seacrowd_sptext":
                        example = {
                            "id": counter,
                            "path": path,
                            "audio": path,
                            "text": sentence,
                            "speaker_id": user_id,
                            "metadata": {
                                "speaker_age": None,
                                "speaker_gender": None,
                            },
                        }
                    yield counter, example
        else:
            for i, path_to_index in enumerate(path_to_indexs):
                geneder = "female" if "female" in path_to_index else "male"
                with open(path_to_index, encoding="utf-8") as f:
                    lines = f.readlines()
                    for id_, line in enumerate(lines):
                        # Following regexs are needed to normalise the lines, since the datasets
                        # are not always consistent and have bugs:
                        line = re.sub(r"\t[^\t]*\t", "\t", line.strip())
                        field_values = re.split(r"\t\t?", line)
                        if len(field_values) != 2:
                            continue
                        filename, sentence = field_values
                        path = os.path.join(path_to_datas[i], f"{filename}.wav")
                        counter += 1
                        if self.config.schema == "source":
                            example = {"path": path, "audio": path, "sentence": sentence}
                        elif self.config.schema == "seacrowd_sptext":
                            example = {
                                "id": counter,
                                "path": path,
                                "audio": path,
                                "text": sentence,
                                "speaker_id": None,
                                "metadata": {
                                    "speaker_age": None,
                                    "speaker_gender": geneder,
                                },
                            }
                        yield counter, example



import os
import json
from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import itertools

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks, Licenses

_CITATION = """\
@inproceedings{ding-etal-2022-globalwoz,
    title = "{G}lobal{W}o{Z}: Globalizing {M}ulti{W}o{Z} to Develop Multilingual Task-Oriented Dialogue Systems",
    author = "Ding, Bosheng  and
      Hu, Junjie  and
      Bing, Lidong  and
      Aljunied, Mahani  and
      Joty, Shafiq  and
      Si, Luo  and
      Miao, Chunyan",
    editor = "Muresan, Smaranda  and
      Nakov, Preslav  and
      Villavicencio, Aline",
    booktitle = "Proceedings of the 60th Annual Meeting of the Association for Computational Linguistics (Volume 1: Long Papers)",
    month = may,
    year = "2022",
}
"""

_DATASETNAME = "globalwoz"

_DESCRIPTION = """\
This is the data of the paper “GlobalWoZ: Globalizing MultiWoZ to Develop Multilingual Task-Oriented Dialogue Systems” accepted by ACL 2022. The dataset contains several sub-datasets in 20 languages and 3 schemes (F&E, E&F, F&F), including Indonesian (id), Thai (th), and Vietnamese (vi) language. The method is based on translating dialogue templates and filling them with local entities in the target language countries.
"""


_HOMEPAGE = "https://github.com/bosheng2020/globalwoz"


_LANGUAGES = ["ind", "tha", "vie"]

_LICENSE = Licenses.UNKNOWN.value

_LOCAL = True

_URLS = {}

_SUPPORTED_TASKS = [Tasks.E2E_TASK_ORIENTED_DIALOGUE]

_SOURCE_VERSION = "2.0.0"

_SEACROWD_VERSION = "1.0.0"


def seacrowd_config_constructor(dial_type, lang, schema, version):
    if dial_type not in ["EandF", "FandE", "FandF"]:
        raise ValueError(f"Invalid dialogue type {dial_type}")

    if lang == "":
        raise ValueError(f"Invalid lang {lang}")

    if schema not in ["source", "seacrowd_tod"]:
        raise ValueError(f"Invalid schema: {schema}")

    return SEACrowdConfig(
        name="globalwoz_{dial_type}_{lang}_{schema}".format(dial_type=dial_type, lang=lang, schema=schema),
        version=datasets.Version(version),
        description="GlobalWoZ schema for {schema}: {dial_type}_{lang}".format(schema=schema, dial_type=dial_type, lang=lang),
        schema=schema,
        subset_id="globalwoz_{dial_type}_{lang}".format(dial_type=dial_type, lang=lang),
    )


class GlobalWoZ(datasets.GeneratorBasedBuilder):
    """This is the data of the paper “GlobalWoZ: Globalizing MultiWoZ to Develop Multilingual Task-Oriented Dialogue Systems” accepted by ACL 2022.
    The dataset contains several sub-datasets in 20 languages and 3 schemes (F&E, E&F, F&F), including Indonesian (id), Thai (th),
    and Vietnamese (vi) language. The method is based on translating dialogue templates and filling them with local entities in the target language countries.
    """

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        seacrowd_config_constructor(tod_format, lang, schema, _SOURCE_VERSION if schema == "source" else _SEACROWD_VERSION) for tod_format, lang, schema in itertools.product(("EandF", "FandE", "FandF"), ("id", "th", "vi"), ("source", "seacrowd_tod"))
    ]

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "goal": {
                        "attraction": datasets.Value("string"),
                        "hospital": datasets.Value("string"),
                        "hotel": datasets.Value("string"),
                        "police": datasets.Value("string"),
                        "restaurant": datasets.Value("string"),
                        "taxi": datasets.Value("string"),
                        "train": datasets.Value("string"),
                    },
                    "log": [
                        {
                            "dialog_act": datasets.Value("string"),
                            "metadata": datasets.Value("string"),
                            "span_info": [[datasets.Value("string")]],
                            "text": datasets.Value("string"),
                        }
                    ],
                }
            )

        elif self.config.schema == "seacrowd_tod":
            features = schemas.tod_features
        else:
            raise NotImplementedError()

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        _split_generators = []

        type_and_lang = {"dial_type": self.config.subset_id.split("_")[1].replace("and", "&"), "lang": self.config.subset_id.split("_")[2]}  # globalwoz_{dial_type}_{lang}

        if self.config.data_dir is None:
            raise ValueError("This is a local dataset. Please pass the data_dir kwarg to load_dataset.")
        else:
            data_dir = self.config.data_dir

        if not os.path.exists(os.path.join(data_dir, f"{type_and_lang['dial_type']}_{type_and_lang['lang']}.json")):
            raise FileNotFoundError()

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    # "filepath": data_dir + f"_{type_and_lang['dial_type']}_{type_and_lang['lang']}.json",
                    "filepath": os.path.join(data_dir, f"{type_and_lang['dial_type']}_{type_and_lang['lang']}.json"),
                    "split": "train",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        # For local datasets you will have access to self.config.data_dir and self.config.data_files
        with open(filepath, "r+", encoding="utf8") as fw:
            data = json.load(fw)

        if self.config.schema == "source":
            for idx, tod_dialogue in enumerate(data.values()):
                example = {}
                example["id"] = str(idx)
                example["goal"] = {}

                for goal_key in ["attraction", "hospital", "hotel", "police", "restaurant", "taxi", "train"]:
                    example["goal"][goal_key] = json.dumps(tod_dialogue["goal"][goal_key])
                example["log"] = []

                for dial_log in tod_dialogue["log"]:
                    dial = {}
                    dial["dialog_act"] = json.dumps(dial_log["dialog_act"])
                    dial["metadata"] = json.dumps(dial_log["metadata"])
                    for i in range(len(dial_log["span_info"])):
                        for j in range(len(dial_log["span_info"][i])):
                            dial_log["span_info"][i][j] = str(dial_log["span_info"][i][j])  # casting to str
                    dial["span_info"] = [[str(span)] if isinstance(span, str) else span for span in dial_log["span_info"]]
                    dial["text"] = dial_log["text"]

                    example["log"].append(dial)

                yield example["id"], example

        elif self.config.schema == "seacrowd_tod":
            for idx, tod_dialogue in enumerate(data.values()):
                example = {}
                example["dialogue_idx"] = idx

                dialogue = []
                # NOTE: the dialogue always started with `user` as first utterance
                for turn, i in enumerate(range(0, len(tod_dialogue["log"]) + 2, 2)):
                    dial = {}
                    dial["turn_idx"] = turn

                    # system_utterance properties
                    dial["system_utterance"] = ""
                    dial["system_acts"] = []
                    if turn != 0:
                        dial["system_utterance"] = tod_dialogue["log"][i - 1]["text"]
                    if i < len(tod_dialogue["log"]):
                        # NOTE: "system_acts will be populated with the `dialog_act` from the user utterance in the original dataset, as our schema dictates
                        # that `system_acts` should represent the system's intended actions based on the user's utterance."
                        for acts in tod_dialogue["log"][i]["dialog_act"].values():
                            for act in acts:
                                dial["system_acts"].append([act[0]])

                    # user_utterance properties
                    dial["turn_label"] = []  # left as an empty array
                    dial["belief_state"] = []
                    if i == len(tod_dialogue["log"]):
                        # case if turn_idx > len(dialogue) --> add dummy user_utterance
                        dial["user_utterance"] = ""
                    else:
                        dial["user_utterance"] = tod_dialogue["log"][i]["text"]
                        # NOTE: "the belief_state will be populated with the `span_info` from the user utterance in the original dataset, as our schema dictates
                        # that `belief_state` should represent the system's belief state based on the user's utterance."
                        for span in tod_dialogue["log"][i]["span_info"]:
                            if span[0].split("-")[1] == "request":  # Request action
                                dial["belief_state"].append({"slots": [["slot", span[1]]], "act": "request"})
                            else:
                                dial["belief_state"].append({"slots": [[span[1], span[2]]], "act": span[0].split("-")[1]})

                    # append to dialogue
                    dialogue.append(dial)

                example["dialogue"] = dialogue

                yield example["dialogue_idx"], example



from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, TASK_TO_SCHEMA, Tasks

_CITATION = """\
@inproceedings{sirihattasak2018annotation,
  title     = {Annotation and Classification of Toxicity for Thai Twitter},
  author    = {Sirihattasak, Sugan and Komachi, Mamoru and Ishikawa, Hiroshi},
  year      = {2018},
  booktitle = {Proceedings of LREC 2018 Workshop and the 2nd Workshop on Text Analytics for Cybersecurity and Online Safety (TA-COS’18)},
  address   = {Miyazaki, Japan},
}
"""

_LOCAL = False
_LANGUAGES = ["tha"]
_DATASETNAME = "thai_toxicity_tweet"
_DESCRIPTION = """\
The Thai Toxicity Tweet Corpus contains 3,300 tweets (506 tweets with texts missing) annotated by humans with guidelines
including a 44-word dictionary. The author acquired 2,027 toxic and 1,273 non-toxic tweets, which were
manually labeled by three annotators. Toxicity is defined by the author as any message conveying harmful,
damaging, or negative intent, in accordance with their defined criteria for toxicity.
"""

_HOMEPAGE = "https://github.com/tmu-nlp/ThaiToxicityTweetCorpus"
_LICENSE = Licenses.CC_BY_NC_4_0.value
_URL = "https://huggingface.co/datasets/thai_toxicity_tweet"

_SUPPORTED_TASKS = [Tasks.ABUSIVE_LANGUAGE_PREDICTION]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class ThaiToxicityTweetsDataset(datasets.GeneratorBasedBuilder):
    """Dataset of Thai tweets annotated for toxicity."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)
    SEACROWD_SCHEMA = TASK_TO_SCHEMA[_SUPPORTED_TASKS[0]].lower()
    CLASS_LABELS = [0, 1]

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=_DATASETNAME,
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_{SEACROWD_SCHEMA}",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema=f"seacrowd_{SEACROWD_SCHEMA}",
            subset_id=_DATASETNAME,
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "tweet_id": datasets.Value("string"),
                    "tweet_text": datasets.Value("string"),
                    "toxic_votes": datasets.Value("int32"),
                    "nontoxic_votes": datasets.Value("int32"),
                    "is_toxic": datasets.ClassLabel(names=self.CLASS_LABELS),
                }
            )

        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA}":
            features = schemas.text_features(label_names=self.CLASS_LABELS)

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        # dl_manager not used since dataloader uses HF `load_dataset`
        """Returns SplitGenerators."""
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"split": "train"},
            ),
        ]

    def _load_hf_data_from_remote(self) -> datasets.DatasetDict:
        """Load dataset from HuggingFace."""
        HF_REMOTE_REF = "/".join(_URL.split("/")[-1:])
        _hf_dataset_source = datasets.load_dataset(HF_REMOTE_REF, split="train")
        return _hf_dataset_source

    def _generate_examples(self, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        data = self._load_hf_data_from_remote()
        index = 0
        for row in data:
            if row["tweet_text"] in ("TWEET_NOT_FOUND", ""):
                continue
            if self.config.schema == "source":
                example = row
            elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA}":
                example = {
                    "id": str(index),
                    "text": row["tweet_text"],
                    "label": row["is_toxic"],
                }
            yield index, example
            index += 1



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
The dataset is split into two:
1. Monolingual (ends with .txt) [Indonesian, Javanese]
2. Bilingual (ends with .tsv) [Indonesian-Javanese, Indonesian-Balinese, Indonesian-Minangkabau, Indonesian-Sundanese]
"""
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@misc{susanto2023replicable,
      title={Replicable Benchmarking of Neural Machine Translation (NMT) on Low-Resource Local Languages in Indonesia},
      author={Lucky Susanto and Ryandito Diandaru and Adila Krisnadhi and Ayu Purwarianti and Derry Wijaya},
      year={2023},
      eprint={2311.00998},
      archivePrefix={arXiv},
      primaryClass={cs.CL}
}
"""
_DATASETNAME = "indonesiannmt"

_DESCRIPTION = """\
This dataset is used on the paper "Replicable Benchmarking of Neural Machine Translation (NMT) on Low-Resource Local Languages in Indonesia". This repository contains two types of data:
1. Monolingual (*.txt) [Indonesian, Javanese]
2. Bilingual (*.tsv) [Indonesian-Javanese, Indonesian-Balinese, Indonesian-Minangkabau, Indonesian-Sundanese]
Only the Bilingual dataset is available for this dataloader
"""

_HOMEPAGE = "https://huggingface.co/datasets/Exqrch/IndonesianNMT"

_LANGUAGES = ["ind", "jav", "ban", "min", "sun"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)

_LICENSE = Licenses.CC_BY_NC_SA_4_0.value

_LOCAL = False

_URLS = {
    "ind_jav": "https://huggingface.co/datasets/Exqrch/IndonesianNMT/resolve/main/id-jv.tsv?download=true",
    "ind_sun": "https://huggingface.co/datasets/Exqrch/IndonesianNMT/resolve/main/id-su.tsv?download=true",
    "ind_ban": "https://huggingface.co/datasets/Exqrch/IndonesianNMT/resolve/main/id-ban.tsv?download=true",
    "ind_min": "https://huggingface.co/datasets/Exqrch/IndonesianNMT/resolve/main/id-min.tsv?download=true",
    "ind": "https://huggingface.co/datasets/Exqrch/IndonesianNMT/resolve/main/bt-id-jv.id.txt?download=true",
    "jav": "https://huggingface.co/datasets/Exqrch/IndonesianNMT/resolve/main/bt-id-jv.jv.txt?download=true",
}

_SUPPORTED_TASKS = [Tasks.MACHINE_TRANSLATION, Tasks.SELF_SUPERVISED_PRETRAINING]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


def seacrowd_config_constructor(modifier, schema, version):
    return SEACrowdConfig(
        name=f"indonesiannmt_{modifier}_{schema}",
        version=version,
        description=f"indonesiannmt_{modifier} {schema} schema",
        schema=f"{schema}",
        subset_id="indonesiannmt",
    )


class IndonesianNMT(datasets.GeneratorBasedBuilder):
    """IndonesianNMT consists of 4 parallel datasets and 2 monolingual datasets,
    all obtained synthetically from either gpt-3.5-turbo or text-davinci-003"""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = (
        [seacrowd_config_constructor(x, "source", _SOURCE_VERSION) for x in ["ind", "jav"]]
        + [seacrowd_config_constructor(x, "seacrowd_ssp", _SOURCE_VERSION) for x in ["ind", "jav"]]
        + [seacrowd_config_constructor(x, "source", _SOURCE_VERSION) for x in ["ind_jav", "ind_min", "ind_sun", "ind_ban"]]
        + [seacrowd_config_constructor(x, "seacrowd_t2t", _SEACROWD_VERSION) for x in ["ind_jav", "ind_min", "ind_sun", "ind_ban"]]
    )

    DEFAULT_CONFIG_NAME = "indonesiannmt_ind_source"

    def is_mono(self):
        if self.config.schema == "seacrowd_ssp":
            return True
        if "source" in self.config.schema:
            if len(self.config.name.split("_")) == 3:
                return True
        return False

    def _info(self) -> datasets.DatasetInfo:
        # ex mono: indonesiannmt_ind_source OR indonesiannmt_ind_seacrowd_ssp
        # ex para: indonesiannmt_ind_jav_source OR indonesiannmt_ind_jav_seacrowd_t2t
        is_mono = self.is_mono()
        if is_mono and self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "text": datasets.Value("string"),
                }
            )
        elif self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "text_1": datasets.Value("string"),
                    "text_2": datasets.Value("string"),
                    "lang_1": datasets.Value("string"),
                    "lang_2": datasets.Value("string"),
                }
            )
        elif self.config.schema == "seacrowd_t2t":
            features = schemas.text_to_text.features
        elif self.config.schema == "seacrowd_ssp":
            features = schemas.self_supervised_pretraining.features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        # ex mono: indonesiannmt_ind_source OR indonesiannmt_ind_seacrowd_ssp
        # ex para: indonesiannmt_ind_jav_source OR indonesiannmt_ind_jav_seacrowd_t2t
        is_mono = self.is_mono()
        if "seacrowd_ssp" in self.config.schema or is_mono:
            lang = self.config.name.split("_")[1]
            path = dl_manager.download_and_extract(_URLS[lang])
        else:
            target = "_".join(self.config.name.split("_")[1:3])
            url = _URLS[target]
            path = dl_manager.download_and_extract(url)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": path,
                    "split": "train",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        is_mono = self.is_mono()
        STR_TO_ISO = {"Indonesian": "ind", "Javanese": "jav", "Minangkabau": "min", "Sundanese": "sun", "Balinese": "ban"}

        with open(filepath, encoding="utf-8") as f:
            flag = True
            if "seacrowd_ssp" in self.config.schema or is_mono:
                for counter, row in enumerate(f):
                    if row.strip != "":
                        yield (
                            counter,
                            {
                                "id": str(counter),
                                "text": row.strip(),
                            },
                        )
            elif self.config.schema == "source":
                for counter, row in enumerate(f):
                    if flag:
                        src, tgt = row.split("\t")
                        tgt = tgt.strip()
                        flag = False
                    else:
                        if row.strip() != "":
                            yield (
                                counter,
                                {
                                    "id": str(counter),
                                    "text_1": row.split("\t")[0].strip(),
                                    "text_2": row.split("\t")[1].strip(),
                                    "lang_1": STR_TO_ISO[src],
                                    "lang_2": STR_TO_ISO[tgt],
                                },
                            )
            elif self.config.schema == "seacrowd_t2t":
                for counter, row in enumerate(f):
                    if flag:
                        src, tgt = row.split("\t")
                        tgt = tgt.strip()
                        flag = False
                    else:
                        if row.strip() != "":
                            yield (
                                counter,
                                {
                                    "id": str(counter),
                                    "text_1": row.split("\t")[0].strip(),
                                    "text_2": row.split("\t")[1].strip(),
                                    "text_1_name": STR_TO_ISO[src],
                                    "text_2_name": STR_TO_ISO[tgt],
                                },
                            )



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
This test is a general test for Malay grammar. Contains 349 questions that may be reinforced with instructions.
"""
import json
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks, Licenses, TASK_TO_SCHEMA

_CITATION = None

_DATASETNAME = "tatabahasa"

_DESCRIPTION = """\
This test is a general test for Malay grammar. Contains 349 questions.
"""

_HOMEPAGE = "https://github.com/mesolitica/malaysian-dataset/tree/master/llm-benchmark/tatabahasabm.tripod.com"

_LANGUAGES = ["zlm"]  

_LICENSE = Licenses.UNLICENSE.value

_LOCAL = False

_URLS = {
    _DATASETNAME: "https://raw.githubusercontent.com/mesolitica/malaysian-dataset/master/llm-benchmark/tatabahasabm.tripod.com/quiz-tatabahasa.jsonl",
}

_SUPPORTED_TASKS = [Tasks.COMMONSENSE_REASONING]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class TatabahasaDataset(datasets.GeneratorBasedBuilder):
    """This test is a general test for Malay grammar. Contains 349 questions."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    SEACROWD_SCHEMA = TASK_TO_SCHEMA[_SUPPORTED_TASKS[0]].lower()

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_{SEACROWD_SCHEMA}",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema=f"seacrowd_{SEACROWD_SCHEMA}",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":
            multi_choice = {"text" : datasets.Value("string"), "answer": datasets.Value("bool")}
            features = datasets.Features({
                "question" : datasets.Value("string"),
                "instruction": datasets.Value("string"),
                "choices": {
                    "A": multi_choice,
                    "B": multi_choice,
                    "C": multi_choice,
                    "D": multi_choice,
                },
                "website": datasets.Value("string")
            })

        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA}":
            features = schemas.qa_features
            features["meta"] = {"website": datasets.Value("string")}

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        urls = _URLS[_DATASETNAME]
        data_dir = dl_manager.download_and_extract(urls)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_dir,
                },
            ),
        ]

    def _generate_examples(self, filepath: Path) -> Tuple[int, Dict]:
        with open(filepath ,'r') as f:
            data = [json.loads(line) for line in f]

        if self.config.schema == "source":
            for i in range(len(data)):
                out = {
                    "question": data[i]["question"],
                    "instruction": data[i]["instruction"],
                    "choices": data[i]["choices"],
                    "website": data[i]["website"]
                }
                yield i, out

        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA}":
            for i in range(len(data)):
                out = {
                    "id": i + 1,
                    "question_id": None,
                    "document_id": None,
                    "question": data[i]["question"],
                    "type": "multiple_choice",
                    "choices": [
                        data[i]["choices"]["A"]["text"],
                        data[i]["choices"]["B"]["text"],
                        data[i]["choices"]["C"]["text"],
                        data[i]["choices"]["D"]["text"],
                    ],
                    "context": data[i]["instruction"],
                    "answer": [choice["text"] for choice in data[i]["choices"].values() if choice["answer"]],
                    "meta": {"website": data[i]["website"]},
                }
                yield i, out



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import csv
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@inproceedings{10.1145/3404835.3463257,
    author = {Srinivasan, Krishna and Raman, Karthik and Chen, Jiecao and Bendersky, Michael and Najork, Marc},
    title = {WIT: Wikipedia-Based Image Text Dataset for Multimodal Multilingual Machine Learning},
    year = {2021},
    isbn = {9781450380379},
    publisher = {Association for Computing Machinery},
    address = {New York, NY, USA},
    url = {https://doi.org/10.1145/3404835.3463257},
    doi = {10.1145/3404835.3463257},
    booktitle = {Proceedings of the 44th International ACM SIGIR Conference on Research and Development in Information Retrieval},
    pages = {2443–2449},
    numpages = {7},
    keywords = {dataset, multimodal, machine learning, wikipedia, multilingual, image-text retrieval, neural networks},
    location = {Virtual Event, Canada},
    series = {SIGIR '21}
}
"""

_DATASETNAME = "wit"

_DESCRIPTION = """\
Wikipedia-based Image Text (WIT) Dataset is a large multimodal multilingual dataset.
WIT is composed of a curated set of 37.6 million entity rich image-text examples with
11.5 million unique images across 108 Wikipedia languages. There are more than 12k
examples in each of 108 languages, with 53 languages having 100k image-text pairs.
Nine languages are spoken in the Southeast Asian region.
Since the dataset contains multiple references, following Section 3.2 of the dataset's
paper, the `seacrowd_imtext` subsets specify which reference is used for each data
instance's texts via context in metadata.
"""

_HOMEPAGE = "https://github.com/google-research-datasets/wit"

_LANGUAGES = {"ceb": "ceb", "fil": "fil", "ind": "id", "jav": "jv", "zlm": "zlm", "mya": "my", "tha": "th", "vie": "vi", "war": "war"}

_LANGUAGE_CODES = list(_LANGUAGES.values())

_LICENSE = Licenses.CC_BY_SA_3_0.value

_LOCAL = False

_URLS = {
    "train_0": "https://storage.googleapis.com/gresearch/wit/wit_v1.train.all-00000-of-00010.tsv.gz",
    "train_1": "https://storage.googleapis.com/gresearch/wit/wit_v1.train.all-00001-of-00010.tsv.gz",
    "train_2": "https://storage.googleapis.com/gresearch/wit/wit_v1.train.all-00002-of-00010.tsv.gz",
    "train_3": "https://storage.googleapis.com/gresearch/wit/wit_v1.train.all-00003-of-00010.tsv.gz",
    "train_4": "https://storage.googleapis.com/gresearch/wit/wit_v1.train.all-00004-of-00010.tsv.gz",
    "train_5": "https://storage.googleapis.com/gresearch/wit/wit_v1.train.all-00005-of-00010.tsv.gz",
    "train_6": "https://storage.googleapis.com/gresearch/wit/wit_v1.train.all-00006-of-00010.tsv.gz",
    "train_7": "https://storage.googleapis.com/gresearch/wit/wit_v1.train.all-00007-of-00010.tsv.gz",
    "train_8": "https://storage.googleapis.com/gresearch/wit/wit_v1.train.all-00008-of-00010.tsv.gz",
    "train_9": "https://storage.googleapis.com/gresearch/wit/wit_v1.train.all-00009-of-00010.tsv.gz",
    "test_0": "https://storage.googleapis.com/gresearch/wit/wit_v1.test.all-00000-of-00005.tsv.gz",
    "test_1": "https://storage.googleapis.com/gresearch/wit/wit_v1.test.all-00001-of-00005.tsv.gz",
    "test_2": "https://storage.googleapis.com/gresearch/wit/wit_v1.test.all-00002-of-00005.tsv.gz",
    "test_3": "https://storage.googleapis.com/gresearch/wit/wit_v1.test.all-00003-of-00005.tsv.gz",
    "test_4": "https://storage.googleapis.com/gresearch/wit/wit_v1.test.all-00004-of-00005.tsv.gz",
    "val_0": "https://storage.googleapis.com/gresearch/wit/wit_v1.val.all-00000-of-00005.tsv.gz",
    "val_1": "https://storage.googleapis.com/gresearch/wit/wit_v1.val.all-00001-of-00005.tsv.gz",
    "val_2": "https://storage.googleapis.com/gresearch/wit/wit_v1.val.all-00002-of-00005.tsv.gz",
    "val_3": "https://storage.googleapis.com/gresearch/wit/wit_v1.val.all-00003-of-00005.tsv.gz",
    "val_4": "https://storage.googleapis.com/gresearch/wit/wit_v1.val.all-00004-of-00005.tsv.gz",
}

_SUPPORTED_TASKS = [Tasks.IMAGE_CAPTIONING]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class WITDataset(datasets.GeneratorBasedBuilder):
    """
    WIT is an image-text dataset from https://huggingface.co/datasets/google/wit.
    """

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = (
        [
            SEACrowdConfig(
                name=f"{_DATASETNAME}_source",
                version=datasets.Version(_SOURCE_VERSION),
                description=f"{_DATASETNAME} source schema for all 9 languages",
                schema="source",
                subset_id=f"{_DATASETNAME}",
            )
        ]
        + [
            SEACrowdConfig(
                name=f"{_DATASETNAME}_seacrowd_imtext",
                version=datasets.Version(_SEACROWD_VERSION),
                description=f"{_DATASETNAME} SEACrowd schema for all 9 languages",
                schema="seacrowd_imtext",
                subset_id=f"{_DATASETNAME}",
            )
        ]
        + [
            SEACrowdConfig(
                name=f"{_DATASETNAME}_{lang}_source",
                version=datasets.Version(_SOURCE_VERSION),
                description=f"{_DATASETNAME}_{lang} source schema",
                schema="source",
                subset_id=f"{_DATASETNAME}_{lang}",
            )
            for lang in _LANGUAGES
        ]
        + [
            SEACrowdConfig(
                name=f"{_DATASETNAME}_{lang}_seacrowd_imtext",
                version=datasets.Version(_SEACROWD_VERSION),
                description=f"{_DATASETNAME}_{lang} SEACrowd schema",
                schema="seacrowd_imtext",
                subset_id=f"{_DATASETNAME}_{lang}",
            )
            for lang in _LANGUAGES
        ]
    )

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "language": datasets.Value("string"),
                    "page_url": datasets.Value("string"),
                    "image_url": datasets.Value("string"),
                    "page_title": datasets.Value("string"),
                    "section_title": datasets.Value("string"),
                    "hierarchical_section_title": datasets.Value("string"),
                    "caption_reference_description": datasets.Value("string"),
                    "caption_attribution_description": datasets.Value("string"),
                    "caption_alt_text_description": datasets.Value("string"),
                    "mime_type": datasets.Value("string"),
                    "original_height": datasets.Value("int32"),
                    "original_width": datasets.Value("int32"),
                    "is_main_image": datasets.Value("bool"),
                    "attribution_passes_lang_id": datasets.Value("bool"),
                    "page_changed_recently": datasets.Value("bool"),
                    "context_page_description": datasets.Value("string"),
                    "context_section_description": datasets.Value("string"),
                }
            )
        elif self.config.schema == "seacrowd_imtext":
            features = schemas.image_text_features()
        else:
            raise ValueError(f"Invalid schema: '{self.config.schema}'")

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """
        Returns SplitGenerators.
        """

        train_paths = dl_manager.download_and_extract([v for k, v in _URLS.items() if "train" in k])
        test_paths = dl_manager.download_and_extract([v for k, v in _URLS.items() if "test" in k])
        val_paths = dl_manager.download_and_extract([v for k, v in _URLS.items() if "val" in k])

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepaths": train_paths,
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepaths": test_paths,
                    "split": "test",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepaths": val_paths,
                    "split": "validation",
                },
            ),
        ]

    def _generate_examples(self, filepaths: Path, split: str) -> Tuple[int, Dict]:
        """
        Yields examples as (key, example) tuples.
        """
        subset_id = self.config.subset_id.split("_")
        if len(subset_id) > 1:
            language_list = subset_id[1]
            if language_list in _LANGUAGES:
                language_list = [_LANGUAGES[language_list]]
        else:
            language_list = _LANGUAGE_CODES

        idx = 0
        for file in filepaths:
            with open(
                file,
                "r",
                encoding="utf-8",
                newline="",
            ) as f:
                data = csv.DictReader(
                    f,
                    delimiter="\t",
                    quoting=csv.QUOTE_ALL,
                )
                if self.config.schema == "seacrowd_imtext":
                    for d in data:
                        if d["language"] in language_list:
                            text = None
                            context = None
                            if d["caption_reference_description"] != "":
                                text = d["caption_reference_description"]
                                context = "caption_reference_description"
                            elif d["caption_attribution_description"] != "":
                                text = d["caption_attribution_description"]
                                context = "caption_attribution_description"
                            else:
                                text = d["caption_alt_text_description"]
                                context = "caption_alt_text_description"
                            x = {
                                "id": idx,
                                "image_paths": [d["image_url"]],
                                "texts": text,
                                "metadata": {
                                    "context": context,
                                    "labels": None,
                                },
                            }
                            yield idx, x
                            idx += 1

                elif self.config.schema == "source":
                    for d in data:
                        if d["language"] in language_list:
                            x = {k: v if v != "" and k in self.info.features else None for k, v in d.items()}
                            yield idx, x
                            idx += 1
                else:
                    raise ValueError(f"Invalid schema: '{self.config.schema}'")
